@@ -1,6 +1,6 @@
 // alloc.cpp -- Memory Allocation Subsystem.
 //
-// $Id: alloc.cpp,v 1.6 2003/07/27 05:21:39 sdennis Exp $
+// $Id: alloc.cpp,v 1.9 2006/01/08 04:26:01 sdennis Exp $
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -52,8 +52,8 @@ typedef struct pooldata
     UINT64 num_lost;                // Buffers lost due to corruption
 } POOL;
 
-POOL pools[NUM_POOLS];
-const char *poolnames[] =
+static POOL pools[NUM_POOLS];
+static const char *poolnames[] =
 {
     "Lbufs", "Sbufs", "Mbufs", "Bools", "Descs", "Qentries", "Pcaches"
 };
@@ -150,7 +150,7 @@ static void pool_vfy(int poolnum, const char *tag, const char *file, const int l
     }
 }
 
-void pool_check(const char *tag, const char *file, const int line)
+static void pool_check(const char *tag, const char *file, const int line)
 {
     int i;
     for (i = 0; i < NUM_POOLS; i++)
@@ -204,7 +204,7 @@ char *pool_alloc(int poolnum, const char *tag, const char *file, const int line)
 
         ph = (POOLHDR *)MEMALLOC(pools[poolnum].pool_size + sizeof(POOLHDR)
            + sizeof(POOLFTR));
-        (void)ISOUTOFMEMORY(ph);
+        ISOUTOFMEMORY(ph);
         p = (char *)(ph + 1);
         pf = (POOLFTR *)(p + pools[poolnum].pool_size);
 
@@ -290,7 +290,7 @@ char *pool_alloc_lbuf(const char *tag, const char *file, const int line)
 
         ph = (POOLHDR *)MEMALLOC(LBUF_SIZE + sizeof(POOLHDR)
            + sizeof(POOLFTR));
-        (void)ISOUTOFMEMORY(ph);
+        ISOUTOFMEMORY(ph);
         p = (char *)(ph + 1);
         pf = (POOLFTR *)(p + LBUF_SIZE);
 
@@ -533,7 +533,7 @@ void list_bufstats(dbref player)
         mux_i64toa(pools[i].max_alloc, szMaxAlloc);
         mux_i64toa(pools[i].tot_alloc, szTotAlloc);
         mux_i64toa(pools[i].num_lost,  szNumLost);
-        
+
         sprintf(buff, "%-12s %5d%10s%10s%14s%7s",
             poolnames[i], (int)pools[i].pool_size,
             szNumAlloc, szMaxAlloc, szTotAlloc, szNumLost);
