@@ -1,6 +1,6 @@
 // conf.cpp -- Set up configuration information and static data.
 //
-// $Id: conf.cpp,v 1.39 2004/04/07 04:37:37 sdennis Exp $
+// $Id: conf.cpp,v 1.70 2006/01/08 03:10:21 sdennis Exp $
 //
 
 #include "copyright.h"
@@ -33,14 +33,6 @@ typedef struct confparm
 CONFDATA mudconf;
 STATEDATA mudstate;
 
-extern NAMETAB logdata_nametab[];
-extern NAMETAB logoptions_nametab[];
-extern NAMETAB access_nametab[];
-extern NAMETAB attraccess_nametab[];
-extern NAMETAB list_names[];
-extern NAMETAB sigactions_nametab[];
-extern CONF conftable[];
-
 // ---------------------------------------------------------------------------
 // cf_init: Initialize mudconf to default values.
 //
@@ -55,6 +47,7 @@ void cf_init(void)
     mudconf.game_pag = StringClone("");
     mudconf.mail_db   = StringClone("mail.db");
     mudconf.comsys_db = StringClone("comsys.db");
+
     mudconf.compress_db = false;
     mudconf.compress = StringClone("gzip");
     mudconf.uncompress = StringClone("gzip -d");
@@ -63,7 +56,7 @@ void cf_init(void)
 
     mudconf.ports.n = 1;
     mudconf.ports.pi = (int *)MEMALLOC(sizeof(int));
-    (void)ISOUTOFMEMORY(mudconf.ports.pi);
+    ISOUTOFMEMORY(mudconf.ports.pi);
     mudconf.ports.pi[0] = 2860;
 
     mudconf.init_size = 1000;
@@ -92,7 +85,9 @@ void cf_init(void)
     mudconf.fixed_home_msg[0] = '\0';
     mudconf.fixed_tel_msg[0] = '\0';
     strcpy(mudconf.public_channel, "Public");
+    strcpy(mudconf.public_channel_alias, "pub");
     strcpy(mudconf.guests_channel, "Guests");
+    strcpy(mudconf.guests_channel_alias, "g");
     strcpy(mudconf.pueblo_msg, "</xch_mudtext><img xch_mode=html>");
     mudconf.art_rules = NULL;
     mudconf.indent_desc = false;
@@ -101,7 +96,10 @@ void cf_init(void)
     mudconf.fork_dump = true;
     mudstate.dumping  = false;
     mudstate.dumper   = 0;
+    mudstate.dumped   = 0;
+    mudstate.write_protect = false;
 #endif
+    mudconf.restrict_home = false;
     mudconf.have_comsys = true;
     mudconf.have_mailer = true;
     mudconf.have_zones = true;
@@ -121,6 +119,17 @@ void cf_init(void)
     mudconf.paycheck = 0;
     mudconf.paystart = 0;
     mudconf.paylimit = 10000;
+#ifdef REALITY_LVLS
+    mudconf.no_levels = 0;
+    mudconf.def_room_rx = 1;
+    mudconf.def_room_tx = ~(RLEVEL)0;
+    mudconf.def_player_rx = 1;
+    mudconf.def_player_tx = 1;
+    mudconf.def_exit_rx = 1;
+    mudconf.def_exit_tx = 1;
+    mudconf.def_thing_rx = 1;
+    mudconf.def_thing_tx = 1;
+#endif /* REALITY_LVLS */
     mudconf.start_quota = 20;
     mudconf.site_chars = 25;
     mudconf.payfind = 0;
@@ -142,43 +151,45 @@ void cf_init(void)
     mudconf.room_quota = 1;
     mudconf.thing_quota = 1;
     mudconf.mail_expiration = 14;
-    mudconf.use_http = false;
     mudconf.queuemax = 100;
     mudconf.queue_chunk = 10;
-    mudconf.active_q_chunk = 10;
-    mudconf.sacfactor = 5;
-    mudconf.sacadjust = -1;
+    mudconf.active_q_chunk  = 10;
+    mudconf.sacfactor       = 5;
+    mudconf.sacadjust       = -1;
+    mudconf.trace_limit     = 200;
+
+    mudconf.autozone        = true;
     mudconf.use_hostname    = true;
-    mudconf.quotas          = false;
-    mudconf.ex_flags        = true;
-    mudconf.robot_speak     = true;
     mudconf.clone_copy_cost = false;
-    mudconf.pub_flags       = true;
-    mudconf.quiet_look      = true;
-    mudconf.exam_public     = true;
-    mudconf.read_rem_desc   = false;
-    mudconf.read_rem_name   = false;
-    mudconf.sweep_dark      = false;
-    mudconf.player_listen   = false;
-    mudconf.quiet_whisper   = true;
     mudconf.dark_sleepers   = true;
-    mudconf.see_own_dark    = true;
+    mudconf.ex_flags        = true;
+    mudconf.exam_public     = true;
+    mudconf.fascist_tport   = false;
     mudconf.idle_wiz_dark   = false;
-    mudconf.pemit_players   = false;
-    mudconf.pemit_any       = false;
     mudconf.match_mine      = true;
     mudconf.match_mine_pl   = true;
-    mudconf.switch_df_all   = true;
-    mudconf.fascist_tport   = false;
-    mudconf.terse_look      = true;
-    mudconf.terse_contents  = true;
-    mudconf.terse_exits     = true;
-    mudconf.terse_movemsg   = true;
-    mudconf.trace_topdown   = true;
-    mudconf.trace_limit     = 200;
+    mudconf.pemit_players   = false;
+    mudconf.pemit_any       = false;
+    mudconf.player_listen   = false;
+    mudconf.pub_flags       = true;
+    mudconf.quiet_look      = true;
+    mudconf.quiet_whisper   = true;
+    mudconf.quotas          = false;
+    mudconf.read_rem_desc   = false;
+    mudconf.read_rem_name   = false;
+    mudconf.reset_players   = false;
+    mudconf.robot_speak     = true;
     mudconf.safe_unowned    = false;
     mudconf.safer_passwords = false;
-    mudconf.autozone        = true;
+    mudconf.see_own_dark    = true;
+    mudconf.sweep_dark      = false;
+    mudconf.switch_df_all   = true;
+    mudconf.terse_contents  = true;
+    mudconf.terse_exits     = true;
+    mudconf.terse_look      = true;
+    mudconf.terse_movemsg   = true;
+    mudconf.trace_topdown   = true;
+    mudconf.use_http        = false;
 
     // -- ??? Running SC on a non-SC DB may cause problems.
     //
@@ -203,10 +214,11 @@ void cf_init(void)
     strcpy(mudconf.mud_name, "MUX");
     strcpy(mudconf.one_coin, "penny");
     strcpy(mudconf.many_coins, "pennies");
-    mudconf.timeslice = 1000;
+    mudconf.timeslice.SetSeconds(1);
     mudconf.cmd_quota_max = 100;
     mudconf.cmd_quota_incr = 1;
-    mudconf.max_cmdsecs.SetSeconds(120);
+    mudconf.rpt_cmdsecs.SetSeconds(120);
+    mudconf.max_cmdsecs.SetSeconds(60);
     mudconf.cache_tick_period.SetSeconds(30);
     mudconf.control_flags = 0xffffffff; // Everything for now...
     mudconf.log_options = LOG_ALWAYS | LOG_BUGS | LOG_SECURITY |
@@ -240,6 +252,9 @@ void cf_init(void)
     mudconf.hook_obj = NOTHING;
     mudconf.global_error_obj = NOTHING;
     mudconf.cache_pages = 40;
+    mudconf.mail_per_hour = 50;
+    mudconf.vattr_per_hour = 5000;
+    mudconf.pcreate_per_hour = 100;
 
     mudstate.events_flag = 0;
     mudstate.bReadingConfiguration = false;
@@ -288,11 +303,12 @@ void cf_init(void)
     mudstate.ntfy_nest_lev = 0;
     mudstate.lock_nest_lev = 0;
     mudstate.zone_nest_num = 0;
+    mudstate.pipe_nest_lev = 0;
     mudstate.inpipe = false;
     mudstate.pout = NULL;
     mudstate.poutnew = NULL;
     mudstate.poutbufc = NULL;
-    mudstate.poutobj = -1;
+    mudstate.poutobj = NOTHING;
     for (i = 0; i < MAX_GLOBAL_REGS; i++)
     {
         mudstate.global_regs[i] = NULL;
@@ -314,13 +330,13 @@ void cf_init(void)
 void cf_log_notfound(dbref player, char *cmd, const char *thingname, char *thing)
 {
     if (mudstate.bReadingConfiguration)
-    {   
+    {
         STARTLOG(LOG_STARTUP, "CNF", "NFND");
         Log.tinyprintf("%s: %s %s not found", cmd, thingname, thing);
         ENDLOG;
     }
     else
-    {   
+    {
         notify(player, tprintf("%s %s not found", thingname, thing));
     }
 }
@@ -354,7 +370,7 @@ void DCL_CDECL cf_log_syntax(dbref player, char *cmd, const char *fmt, ...)
 // ---------------------------------------------------------------------------
 // cf_status_from_succfail: Return command status from succ and fail info
 //
-int cf_status_from_succfail(dbref player, char *cmd, int success, int failure)
+static int cf_status_from_succfail(dbref player, char *cmd, int success, int failure)
 {
     char *buff;
 
@@ -386,13 +402,49 @@ int cf_status_from_succfail(dbref player, char *cmd, int success, int failure)
     return -1;
 }
 
+//---------------------------------------------------------------------------
+// cf_rlevel
+//
+
+#ifdef REALITY_LVLS
+
+CF_HAND(cf_rlevel)
+{
+    CONFDATA *mc = (CONFDATA *)vp;
+    int i;
+
+    if(mc->no_levels >= 32)
+        return 1;
+    for(i=0; *str && !mux_isspace[*str]; ++str)
+        if(i < 8)
+            mc->reality_level[mc->no_levels].name[i++] = *str;
+    mc->reality_level[mc->no_levels].name[i] = '\0';
+    mc->reality_level[mc->no_levels].value = 1;
+    strcpy(mc->reality_level[mc->no_levels].attr, "DESC");
+    for(; *str && mux_isspace[*str]; ++str);
+    for(i=0; *str && mux_isdigit[*str]; ++str)
+        i = i * 10 + (*str - '0');
+    if(i)
+        mc->reality_level[mc->no_levels].value = (RLEVEL) i;
+    for(; *str && mux_isspace[*str]; ++str);
+    if(*str)
+        strncpy(mc->reality_level[mc->no_levels].attr, str, 32);
+    mc->no_levels++;
+    return 0;
+}
+#endif /* REALITY_LVLS */
+
 // ---------------------------------------------------------------------------
 // cf_int_array: Setup array of integers.
 //
-CF_HAND(cf_int_array)
+static CF_HAND(cf_int_array)
 {
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(player);
+    UNUSED_PARAMETER(cmd);
+
     int *aPorts = (int *)MEMALLOC(nExtra*sizeof(int));
-    (void)ISOUTOFMEMORY(aPorts);
+    ISOUTOFMEMORY(aPorts);
     unsigned int nPorts = 0;
 
     char *p;
@@ -421,7 +473,7 @@ CF_HAND(cf_int_array)
             pia->pi = NULL;
         }
         pia->pi = (int *)MEMALLOC(nPorts * sizeof(int));
-        (void)ISOUTOFMEMORY(pia->pi);
+        ISOUTOFMEMORY(pia->pi);
         pia->n = nPorts;
         for (unsigned int i = 0; i < nPorts; i++)
         {
@@ -435,8 +487,13 @@ CF_HAND(cf_int_array)
 // ---------------------------------------------------------------------------
 // cf_int: Set integer parameter.
 //
-CF_HAND(cf_int)
+static CF_HAND(cf_int)
 {
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+    UNUSED_PARAMETER(player);
+    UNUSED_PARAMETER(cmd);
+
     // Copy the numeric value to the parameter.
     //
     *vp = mux_atol(str);
@@ -446,8 +503,13 @@ CF_HAND(cf_int)
 // ---------------------------------------------------------------------------
 // cf_dbref: Set dbref parameter....looking for an ignoring the leading '#'.
 //
-CF_HAND(cf_dbref)
+static CF_HAND(cf_dbref)
 {
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+    UNUSED_PARAMETER(player);
+    UNUSED_PARAMETER(cmd);
+
     char *p = str;
     while (mux_isspace(*p))
     {
@@ -467,8 +529,13 @@ CF_HAND(cf_dbref)
 // ---------------------------------------------------------------------------
 // cf_seconds: Set CLinearTimeDelta in units of seconds.
 //
-CF_HAND(cf_seconds)
+static CF_HAND(cf_seconds)
 {
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+    UNUSED_PARAMETER(player);
+    UNUSED_PARAMETER(cmd);
+
     CLinearTimeDelta *pltd = (CLinearTimeDelta *)vp;
     pltd->SetSecondsString(str);
     return 0;
@@ -477,7 +544,7 @@ CF_HAND(cf_seconds)
 // ---------------------------------------------------------------------------
 // cf_bool: Set boolean parameter.
 //
-NAMETAB bool_names[] =
+static NAMETAB bool_names[] =
 {
     {"true",    1,  0,  true},
     {"false",   1,  0,  false},
@@ -488,10 +555,13 @@ NAMETAB bool_names[] =
     {NULL,      0,  0,  0}
 };
 
-CF_HAND(cf_bool)
+static CF_HAND(cf_bool)
 {
-    int i = search_nametab(GOD, bool_names, str);
-    if (i < 0)
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+
+    int i;
+    if (!search_nametab(GOD, bool_names, str, &i))
     {
         cf_log_notfound(player, cmd, "Value", str);
         return -1;
@@ -504,10 +574,12 @@ CF_HAND(cf_bool)
 // ---------------------------------------------------------------------------
 // cf_option: Select one option from many choices.
 //
-CF_HAND(cf_option)
+static CF_HAND(cf_option)
 {
-    int i = search_nametab(GOD, (NAMETAB *) pExtra, str);
-    if (i < 0)
+    UNUSED_PARAMETER(nExtra);
+
+    int i;
+    if (!search_nametab(GOD, (NAMETAB *)pExtra, str, &i))
     {
         cf_log_notfound(player, cmd, "Value", str);
         return -1;
@@ -519,8 +591,10 @@ CF_HAND(cf_option)
 // ---------------------------------------------------------------------------
 // cf_string: Set string parameter.
 //
-CF_HAND(cf_string)
+static CF_HAND(cf_string)
 {
+    UNUSED_PARAMETER(pExtra);
+
     char *pc = (char *)vp;
 
     // The following should never happen because extra is always a non-zero
@@ -562,14 +636,15 @@ CF_HAND(cf_string)
         char *p = buff;
         char *q = strip_ansi(mudconf.mud_name);
         size_t nLen = 0;
-        while (*q && nLen < SBUF_SIZE)
+        while (  *q
+              && nLen < SBUF_SIZE)
         {
             if (mux_isalnum(*q))
             {
                 *p++ = *q;
                 nLen++;
             }
-            *q++;
+            q++;
         }
         *p = '\0';
         Log.SetPrefix(buff);
@@ -581,8 +656,10 @@ CF_HAND(cf_string)
 // ---------------------------------------------------------------------------
 // cf_string_dyn: Set string parameter using dynamically allocated memory.
 //
-CF_HAND(cf_string_dyn)
+static CF_HAND(cf_string_dyn)
 {
+    UNUSED_PARAMETER(pExtra);
+
     char **ppc = (char **)vp;
 
     // Allocate memory for buffer and copy string to it. If nExtra is non-zero,
@@ -621,8 +698,11 @@ CF_HAND(cf_string_dyn)
 // ---------------------------------------------------------------------------
 // cf_alias: define a generic hash table alias.
 //
-CF_HAND(cf_alias)
+static CF_HAND(cf_alias)
 {
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+
     MUX_STRTOK_STATE tts;
     mux_strtok_src(&tts, str);
     mux_strtok_ctl(&tts, " \t=,");
@@ -643,7 +723,10 @@ CF_HAND(cf_alias)
                 return -1;
             }
         }
-        hashaddLEN(alias, strlen(alias), cp, (CHashTable *) vp);
+        if (!hashfindLEN(alias, strlen(alias), (CHashTable *) vp))
+        {
+            hashaddLEN(alias, strlen(alias), cp, (CHashTable *) vp);
+        }
         return 0;
     }
     return -1;
@@ -652,8 +735,12 @@ CF_HAND(cf_alias)
 // ---------------------------------------------------------------------------
 // cf_flagalias: define a flag alias.
 //
-CF_HAND(cf_flagalias)
+static CF_HAND(cf_flagalias)
 {
+    UNUSED_PARAMETER(vp);
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+
     MUX_STRTOK_STATE tts;
     mux_strtok_src(&tts, str);
     mux_strtok_ctl(&tts, " \t=,");
@@ -665,14 +752,20 @@ CF_HAND(cf_flagalias)
     bool bValid;
     void *cp;
     char *pName = MakeCanonicalFlagName(orig, &nName, &bValid);
-    if (  bValid
-       && (cp = hashfindLEN(pName, nName, &mudstate.flags_htab)))
+    if (bValid)
     {
-        pName = MakeCanonicalFlagName(alias, &nName, &bValid);
-        if (bValid)
+        cp = hashfindLEN(pName, nName, &mudstate.flags_htab);
+        if (cp)
         {
-            hashaddLEN(pName, nName, cp, &mudstate.flags_htab);
-            success = true;
+            pName = MakeCanonicalFlagName(alias, &nName, &bValid);
+            if (bValid)
+            {
+                if (!hashfindLEN(pName, nName, &mudstate.flags_htab))
+                {
+                    hashaddLEN(pName, nName, cp, &mudstate.flags_htab);
+                    success = true;
+               }
+            }
         }
     }
     if (!success)
@@ -685,8 +778,12 @@ CF_HAND(cf_flagalias)
 // ---------------------------------------------------------------------------
 // cf_poweralias: define a power alias.
 //
-CF_HAND(cf_poweralias)
+static CF_HAND(cf_poweralias)
 {
+    UNUSED_PARAMETER(vp);
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+
     MUX_STRTOK_STATE tts;
     mux_strtok_src(&tts, str);
     mux_strtok_ctl(&tts, " \t=,");
@@ -698,14 +795,17 @@ CF_HAND(cf_poweralias)
     bool bValid;
     void *cp;
     char *pName = MakeCanonicalFlagName(orig, &nName, &bValid);
-    if (  bValid
-       && (cp = hashfindLEN(pName, nName, &mudstate.powers_htab)))
+    if (bValid)
     {
-        pName = MakeCanonicalFlagName(alias, &nName, &bValid);
-        if (bValid)
+        cp = hashfindLEN(pName, nName, &mudstate.powers_htab);
+        if (cp)
         {
-            hashaddLEN(pName, nName, cp, &mudstate.powers_htab);
-            success = true;
+            pName = MakeCanonicalFlagName(alias, &nName, &bValid);
+            if (bValid)
+            {
+                hashaddLEN(pName, nName, cp, &mudstate.powers_htab);
+                success = true;
+            }
         }
     }
     if (!success)
@@ -715,11 +815,14 @@ CF_HAND(cf_poweralias)
     return (success ? 0 : -1);
 }
 
+#if 0
 // ---------------------------------------------------------------------------
 // cf_or_in_bits: OR in bits from namelist to a word.
 //
-CF_HAND(cf_or_in_bits)
+static CF_HAND(cf_or_in_bits)
 {
+    UNUSED_PARAMETER(nExtra);
+
     int f, success, failure;
 
     // Walk through the tokens.
@@ -733,8 +836,7 @@ CF_HAND(cf_or_in_bits)
     {
         // Set the appropriate bit.
         //
-        f = search_nametab(GOD, (NAMETAB *) pExtra, sp);
-        if (f > 0)
+        if (search_nametab(GOD, (NAMETAB *)pExtra, sp, &f))
         {
             *vp |= f;
             success++;
@@ -751,12 +853,15 @@ CF_HAND(cf_or_in_bits)
     }
     return cf_status_from_succfail(player, cmd, success, failure);
 }
+#endif
 
 // ---------------------------------------------------------------------------
 // cf_modify_bits: set or clear bits in a flag word from a namelist.
 //
 CF_HAND(cf_modify_bits)
 {
+    UNUSED_PARAMETER(nExtra);
+
     int f, success, failure;
     bool negate;
 
@@ -780,8 +885,7 @@ CF_HAND(cf_modify_bits)
 
         // Set or clear the appropriate bit.
         //
-        f = search_nametab(GOD, (NAMETAB *)pExtra, sp);
-        if (f > 0)
+        if (search_nametab(GOD, (NAMETAB *)pExtra, sp, &f))
         {
             if (negate)
                 *vp &= ~f;
@@ -802,11 +906,14 @@ CF_HAND(cf_modify_bits)
     return cf_status_from_succfail(player, cmd, success, failure);
 }
 
+#if 0
 // ---------------------------------------------------------------------------
 // cf_set_bits: Clear flag word and then set specified bits from namelist.
 //
-CF_HAND(cf_set_bits)
+static CF_HAND(cf_set_bits)
 {
+    UNUSED_PARAMETER(nExtra);
+
     int f, success, failure;
 
     // Walk through the tokens
@@ -822,8 +929,7 @@ CF_HAND(cf_set_bits)
     {
         // Set the appropriate bit.
         //
-        f = search_nametab(GOD, (NAMETAB *)pExtra, sp);
-        if (f > 0)
+        if (search_nametab(GOD, (NAMETAB *)pExtra, sp, &f))
         {
             *vp |= f;
             success++;
@@ -840,12 +946,16 @@ CF_HAND(cf_set_bits)
     }
     return cf_status_from_succfail(player, cmd, success, failure);
 }
+#endif
 
 // ---------------------------------------------------------------------------
 // cf_set_flags: Clear flag word and then set from a flags htab.
 //
-CF_HAND(cf_set_flags)
+static CF_HAND(cf_set_flags)
 {
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+
     int success, failure;
 
     // Walk through the tokens.
@@ -919,12 +1029,21 @@ CF_HAND(cf_set_flags)
 // ---------------------------------------------------------------------------
 // cf_badname: Disallow use of player name/alias.
 //
-CF_HAND(cf_badname)
+static CF_HAND(cf_badname)
 {
+    UNUSED_PARAMETER(vp);
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(player);
+    UNUSED_PARAMETER(cmd);
+
     if (nExtra)
+    {
         badname_remove(str);
+    }
     else
+    {
         badname_add(str);
+    }
     return 0;
 }
 
@@ -1162,7 +1281,7 @@ static bool MakeCanonicalIPv4(const char *str, in_addr_t *pnIP)
 // valid one. Valid masks consist of a N-bit sequence of '1' bits followed by
 // a (32-N)-bit sequence of '0' bits, where N is 0 to 32.
 //
-bool isValidSubnetMask(in_addr_t ulMask)
+static bool isValidSubnetMask(in_addr_t ulMask)
 {
     in_addr_t ulTest = 0xFFFFFFFFUL;
     for (int i = 0; i <= 32; i++)
@@ -1179,8 +1298,10 @@ bool isValidSubnetMask(in_addr_t ulMask)
 // ---------------------------------------------------------------------------
 // cf_site: Update site information
 
-CF_HAND(cf_site)
+static CF_HAND(cf_site)
 {
+    UNUSED_PARAMETER(pExtra);
+
     SITE **ppv = (SITE **)vp;
     struct in_addr addr_num, mask_num;
     in_addr_t ulMask, ulNetBits;
@@ -1268,7 +1389,7 @@ CF_HAND(cf_site)
     // Parse the access entry and allocate space for it.
     //
     SITE *site = (SITE *)MEMALLOC(sizeof(SITE));
-    (void)ISOUTOFMEMORY(site);
+    ISOUTOFMEMORY(site);
 
     // Initialize the site entry.
     //
@@ -1306,51 +1427,11 @@ CF_HAND(cf_site)
     return 0;
 }
 
-// ---------------------------------------------------------------------------
-// cf_cf_access: Set access on config directives
-//
-CF_HAND(cf_cf_access)
-{
-    CONF *tp;
-    char *ap;
-
-    for (ap = str; *ap && !mux_isspace(*ap); ap++)
-    {
-        ; // Nothing
-    }
-    if (*ap)
-    {
-        *ap++ = '\0';
-    }
-
-    for (tp = conftable; tp->pname; tp++)
-    {
-        if (!strcmp(tp->pname, str))
-        {
-            // Cannot modify parameters set CA_STATIC.
-            //
-            if (tp->flags & CA_STATIC)
-            {
-                notify(player, NOPERM_MESSAGE);
-                STARTLOG(LOG_CONFIGMODS, "CFG", "PERM");
-                log_name(player);
-                log_text(" tried to change access to static param: ");
-                log_text(tp->pname);
-                ENDLOG;
-                return -1;
-            }
-            return cf_modify_bits(&tp->flags, ap, pExtra, nExtra, player, cmd);
-        }
-    }
-    cf_log_notfound(player, cmd, "Config directive", str);
-    return -1;
-}
-
 // ---------------------------------------------------------------------------
 // cf_helpfile, cf_raw_helpfile: Add help files and their corresponding
 // command.
 //
-int add_helpfile(dbref player, char *cmd, char *str, bool bRaw)
+static int add_helpfile(dbref player, char *cmd, char *str, bool bRaw)
 {
     // Parse the two arguments.
     //
@@ -1387,13 +1468,13 @@ int add_helpfile(dbref player, char *cmd, char *str, bool bRaw)
         mudstate.nHelpDesc = 0;
         mudstate.aHelpDesc = (HELP_DESC *)MEMALLOC(sizeof(HELP_DESC)
             *mudstate.mHelpDesc);
-        (void)ISOUTOFMEMORY(mudstate.aHelpDesc);
+        ISOUTOFMEMORY(mudstate.aHelpDesc);
     }
     else if (mudstate.mHelpDesc <= mudstate.nHelpDesc)
     {
         int newsize = mudstate.mHelpDesc + 4;
         HELP_DESC *q = (HELP_DESC *)MEMALLOC(sizeof(HELP_DESC)*newsize);
-        (void)ISOUTOFMEMORY(q);
+        ISOUTOFMEMORY(q);
         memset(q, 0, sizeof(HELP_DESC)*newsize);
         memcpy(q, mudstate.aHelpDesc, sizeof(HELP_DESC)*mudstate.mHelpDesc);
         MEMFREE(mudstate.aHelpDesc);
@@ -1412,7 +1493,7 @@ int add_helpfile(dbref player, char *cmd, char *str, bool bRaw)
     // Build up Command Entry.
     //
     CMDENT_ONE_ARG *cmdp = (CMDENT_ONE_ARG *)MEMALLOC(sizeof(CMDENT_ONE_ARG));
-    (void)ISOUTOFMEMORY(cmdp);
+    ISOUTOFMEMORY(cmdp);
 
     cmdp->callseq = CS_ONE_ARG;
     cmdp->cmdname = StringClone(pCmdName);
@@ -1439,13 +1520,21 @@ int add_helpfile(dbref player, char *cmd, char *str, bool bRaw)
     return 0;
 }
 
-CF_HAND(cf_helpfile)
+static CF_HAND(cf_helpfile)
 {
+    UNUSED_PARAMETER(vp);
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+
     return add_helpfile(player, cmd, str, false);
 }
 
-CF_HAND(cf_raw_helpfile)
+static CF_HAND(cf_raw_helpfile)
 {
+    UNUSED_PARAMETER(vp);
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+
     return add_helpfile(player, cmd, str, true);
 }
 
@@ -1454,7 +1543,7 @@ CF_HAND(cf_raw_helpfile)
 // Used with express permission of RhostMUSH developers.
 // Bludgeoned into MUX by Jake Nelson 7/2002.
 //
-NAMETAB hook_names[] =
+static NAMETAB hook_names[] =
 {
     {"after",      3, 0, HOOK_AFTER},
     {"before",     3, 0, HOOK_BEFORE},
@@ -1465,8 +1554,13 @@ NAMETAB hook_names[] =
     {NULL,         0, 0, 0}
 };
 
-CF_HAND(cf_hook)
+static CF_HAND(cf_hook)
 {
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+    UNUSED_PARAMETER(player);
+    UNUSED_PARAMETER(cmd);
+
     char *hookcmd, *hookptr, playbuff[201];
     int hookflg;
     CMDENT *cmdp;
@@ -1496,11 +1590,10 @@ CF_HAND(cf_hook)
     hookptr = mux_strtok_parse(&tts);
     while (hookptr != NULL)
     {
-       if (  *hookptr == '!'
-          && *(hookptr + 1))
+       if (  hookptr[0] == '!'
+          && hookptr[1] != '\0')
        {
-          hookflg = search_nametab(GOD, hook_names, hookptr + 1);
-          if (0 < hookflg)
+          if (search_nametab(GOD, hook_names, hookptr+1, &hookflg))
           {
              retval = 0;
              *vp = *vp & ~hookflg;
@@ -1508,8 +1601,7 @@ CF_HAND(cf_hook)
        }
        else
        {
-          hookflg = search_nametab(GOD, hook_names, hookptr);
-          if (0 < hookflg)
+          if (search_nametab(GOD, hook_names, hookptr, &hookflg))
           {
              retval = 0;
              *vp = *vp | hookflg;
@@ -1524,9 +1616,11 @@ CF_HAND(cf_hook)
 // ---------------------------------------------------------------------------
 // cf_include: Read another config file.  Only valid during startup.
 //
-CF_HAND(cf_include)
+static CF_HAND(cf_include)
 {
-    extern int cf_set(char *, char *, dbref);
+    UNUSED_PARAMETER(vp);
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
 
     if (!mudstate.bReadingConfiguration)
     {
@@ -1578,7 +1672,7 @@ CF_HAND(cf_include)
         // Process line.
         //
         char *cp = buf;
-        
+
         // Trim leading spaces.
         //
         while (mux_isspace(*cp))
@@ -1622,19 +1716,10 @@ CF_HAND(cf_include)
     return 0;
 }
 
-extern CF_HAND(cf_access);
-extern CF_HAND(cf_cmd_alias);
-extern CF_HAND(cf_acmd_access);
-extern CF_HAND(cf_attr_access);
-extern CF_HAND(cf_func_access);
-extern CF_HAND(cf_flag_access);
-extern CF_HAND(cf_flag_name);
-extern CF_HAND(cf_art_rule);
-
 // ---------------------------------------------------------------------------
 // conftable: Table for parsing the configuration file.
 
-CONF conftable[] =
+static CONF conftable[] =
 {
     {"access",                    cf_access,      CA_GOD,    CA_DISABLED, NULL,                            access_nametab,     0},
     {"alias",                     cf_cmd_alias,   CA_GOD,    CA_DISABLED, (int *)&mudstate.command_htab,   0,                  0},
@@ -1661,6 +1746,7 @@ CONF conftable[] =
     {"conn_timeout",              cf_int,         CA_GOD,    CA_WIZARD,   &mudconf.conn_timeout,           NULL,               0},
     {"connect_file",              cf_string_dyn,  CA_STATIC, CA_GOD,      (int *)&mudconf.conn_file,       NULL, SIZEOF_PATHNAME},
     {"connect_reg_file",          cf_string_dyn,  CA_STATIC, CA_GOD,      (int *)&mudconf.creg_file,       NULL, SIZEOF_PATHNAME},
+    {"lag_limit",                 cf_seconds,     CA_GOD,    CA_WIZARD,   (int *)&mudconf.max_cmdsecs,     NULL,               0},
     {"crash_database",            cf_string_dyn,  CA_STATIC, CA_GOD,      (int *)&mudconf.crashdb,         NULL, SIZEOF_PATHNAME},
     {"create_max_cost",           cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.createmax,              NULL,               0},
     {"create_min_cost",           cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.createmin,              NULL,               0},
@@ -1707,6 +1793,7 @@ CONF conftable[] =
     {"guest_prefix",              cf_string,      CA_STATIC, CA_PUBLIC,   (int *)mudconf.guest_prefix,     NULL,              32},
     {"guest_site",                cf_site,        CA_GOD,    CA_DISABLED, (int *)&mudstate.access_list,    NULL,         H_GUEST},
     {"guests_channel",            cf_string,      CA_STATIC, CA_PUBLIC,   (int *)mudconf.guests_channel,   NULL,              32},
+    {"guests_channel_alias",      cf_string,      CA_STATIC, CA_PUBLIC,   (int *)mudconf.guests_channel_alias, NULL,          32},
     {"have_comsys",               cf_bool,        CA_STATIC, CA_PUBLIC,   (int *)&mudconf.have_comsys,     NULL,               0},
     {"have_mailer",               cf_bool,        CA_STATIC, CA_PUBLIC,   (int *)&mudconf.have_mailer,     NULL,               0},
     {"have_zones",                cf_bool,        CA_STATIC, CA_PUBLIC,   (int *)&mudconf.have_zones,      NULL,               0},
@@ -1724,7 +1811,7 @@ CONF conftable[] =
     {"kill_guarantee_cost",       cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.killguarantee,          NULL,               0},
     {"kill_max_cost",             cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.killmax,                NULL,               0},
     {"kill_min_cost",             cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.killmin,                NULL,               0},
-    {"lag_maximum",               cf_seconds,     CA_GOD,    CA_WIZARD,   (int *)&mudconf.max_cmdsecs,     NULL,               0},
+    {"lag_maximum",               cf_seconds,     CA_GOD,    CA_WIZARD,   (int *)&mudconf.rpt_cmdsecs,     NULL,               0},
     {"link_cost",                 cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.linkcost,               NULL,               0},
     {"list_access",               cf_ntab_access, CA_GOD,    CA_DISABLED, (int *)list_names,               access_nametab,     0},
     {"lock_recursion_limit",      cf_int,         CA_WIZARD, CA_PUBLIC,   &mudconf.lock_nest_lim,          NULL,               0},
@@ -1736,6 +1823,7 @@ CONF conftable[] =
     {"machine_command_cost",      cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.machinecost,            NULL,               0},
     {"mail_database",             cf_string_dyn,  CA_GOD,    CA_GOD,      (int *)&mudconf.mail_db,         NULL, SIZEOF_PATHNAME},
     {"mail_expiration",           cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.mail_expiration,        NULL,               0},
+    {"mail_per_hour",             cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.mail_per_hour,          NULL,               0},
     {"master_room",               cf_dbref,       CA_GOD,    CA_WIZARD,   &mudconf.master_room,            NULL,               0},
     {"match_own_commands",        cf_bool,        CA_GOD,    CA_PUBLIC,   (int *)&mudconf.match_mine,      NULL,               0},
     {"max_cache_size",            cf_int,         CA_GOD,    CA_GOD,      (int *)&mudconf.max_cache_size,  NULL,               0},
@@ -1771,7 +1859,9 @@ CONF conftable[] =
     {"port",                      cf_int_array,   CA_STATIC, CA_PUBLIC,   (int *)&mudconf.ports,           NULL, MAX_LISTEN_PORTS},
     {"postdump_message",          cf_string,      CA_GOD,    CA_WIZARD,   (int *)mudconf.postdump_msg,     NULL,             128},
     {"power_alias",               cf_poweralias,  CA_GOD,    CA_DISABLED, NULL,                            NULL,               0},
+    {"pcreate_per_hour",          cf_int,         CA_STATIC, CA_PUBLIC,   (int *)&mudconf.pcreate_per_hour,NULL,               0},
     {"public_channel",            cf_string,      CA_STATIC, CA_PUBLIC,   (int *)mudconf.public_channel,   NULL,              32},
+    {"public_channel_alias",      cf_string,      CA_STATIC, CA_PUBLIC,   (int *)mudconf.public_channel_alias, NULL,          32},
     {"public_flags",              cf_bool,        CA_GOD,    CA_PUBLIC,   (int *)&mudconf.pub_flags,       NULL,               0},
     {"pueblo_message",            cf_string,      CA_GOD,    CA_WIZARD,   (int *)mudconf.pueblo_msg,       NULL,       GBUF_SIZE},
     {"queue_active_chunk",        cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.active_q_chunk,         NULL,               0},
@@ -1785,6 +1875,8 @@ CONF conftable[] =
     {"read_remote_name",          cf_bool,        CA_GOD,    CA_PUBLIC,   (int *)&mudconf.read_rem_name,   NULL,               0},
     {"register_create_file",      cf_string_dyn,  CA_STATIC, CA_GOD,      (int *)&mudconf.regf_file,       NULL, SIZEOF_PATHNAME},
     {"register_site",             cf_site,        CA_GOD,    CA_DISABLED, (int *)&mudstate.access_list,    NULL,  H_REGISTRATION},
+    {"reset_players",             cf_bool,        CA_GOD,    CA_DISABLED, (int *)&mudconf.reset_players,   NULL,               0},
+    {"restrict_home",             cf_bool,        CA_GOD,    CA_DISABLED, (int *)&mudconf.restrict_home,   NULL,               0},
     {"retry_limit",               cf_int,         CA_GOD,    CA_WIZARD,   &mudconf.retry_limit,            NULL,               0},
     {"robot_cost",                cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.robotcost,              NULL,               0},
     {"robot_flags",               cf_set_flags,   CA_GOD,    CA_DISABLED, (int *)&mudconf.robot_flags,     NULL,               0},
@@ -1813,7 +1905,7 @@ CONF conftable[] =
     {"terse_shows_move_messages", cf_bool,        CA_GOD,    CA_PUBLIC,   (int *)&mudconf.terse_movemsg,   NULL,               0},
     {"thing_flags",               cf_set_flags,   CA_GOD,    CA_DISABLED, (int *)&mudconf.thing_flags,     NULL,               0},
     {"thing_quota",               cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.thing_quota,            NULL,               0},
-    {"timeslice",                 cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.timeslice,              NULL,               0},
+    {"timeslice",                 cf_seconds,     CA_GOD,    CA_PUBLIC,   (int *)&mudconf.timeslice,       NULL,               0},
     {"toad_recipient",            cf_dbref,       CA_GOD,    CA_WIZARD,   &mudconf.toad_recipient,         NULL,               0},
     {"trace_output_limit",        cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.trace_limit,            NULL,               0},
     {"trace_topdown",             cf_bool,        CA_GOD,    CA_PUBLIC,   (int *)&mudconf.trace_topdown,   NULL,               0},
@@ -1822,13 +1914,68 @@ CONF conftable[] =
     {"unowned_safe",              cf_bool,        CA_GOD,    CA_PUBLIC,   (int *)&mudconf.safe_unowned,    NULL,               0},
     {"use_http",                  cf_bool,        CA_STATIC, CA_PUBLIC,   (int *)&mudconf.use_http,        NULL,               0},
     {"user_attr_access",          cf_modify_bits, CA_GOD,    CA_DISABLED, &mudconf.vattr_flags,            attraccess_nametab, 0},
+    {"user_attr_per_hour",        cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.vattr_per_hour,         NULL,               0},
     {"wait_cost",                 cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.waitcost,               NULL,               0},
     {"wizard_motd_file",          cf_string_dyn,  CA_STATIC, CA_GOD,      (int *)&mudconf.wizmotd_file,    NULL, SIZEOF_PATHNAME},
     {"wizard_motd_message",       cf_string,      CA_GOD,    CA_WIZARD,   (int *)mudconf.wizmotd_msg,      NULL,       GBUF_SIZE},
     {"zone_recursion_limit",      cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.zone_nest_lim,          NULL,               0},
+#ifdef REALITY_LVLS
+    {"reality_level",             cf_rlevel,      CA_STATIC, CA_GOD,      (int *)&mudconf,                 NULL,               0},
+    {"def_room_rx",               cf_int,         CA_WIZARD, CA_PUBLIC,   (int *)&mudconf.def_room_rx,     NULL,               0},
+    {"def_room_tx",               cf_int,         CA_WIZARD, CA_PUBLIC,   (int *)&mudconf.def_room_tx,     NULL,               0},
+    {"def_player_rx",             cf_int,         CA_WIZARD, CA_PUBLIC,   (int *)&mudconf.def_player_rx,   NULL,               0},
+    {"def_player_tx",             cf_int,         CA_WIZARD, CA_PUBLIC,   (int *)&mudconf.def_player_tx,   NULL,               0},
+    {"def_exit_rx",               cf_int,         CA_WIZARD, CA_PUBLIC,   (int *)&mudconf.def_exit_rx,     NULL,               0},
+    {"def_exit_tx",               cf_int,         CA_WIZARD, CA_PUBLIC,   (int *)&mudconf.def_exit_tx,     NULL,               0},
+    {"def_thing_rx",              cf_int,         CA_WIZARD, CA_PUBLIC,   (int *)&mudconf.def_thing_rx,    NULL,               0},
+    {"def_thing_tx",              cf_int,         CA_WIZARD, CA_PUBLIC,   (int *)&mudconf.def_thing_tx,    NULL,               0},
+#endif /* REALITY_LVLS */
     { NULL,                       NULL,           0,         0,           NULL,                            NULL,               0}
 };
 
+// ---------------------------------------------------------------------------
+// cf_cf_access: Set access on config directives
+//
+CF_HAND(cf_cf_access)
+{
+    UNUSED_PARAMETER(vp);
+
+    CONF *tp;
+    char *ap;
+
+    for (ap = str; *ap && !mux_isspace(*ap); ap++)
+    {
+        ; // Nothing
+    }
+    if (*ap)
+    {
+        *ap++ = '\0';
+    }
+
+    for (tp = conftable; tp->pname; tp++)
+    {
+        if (!strcmp(tp->pname, str))
+        {
+            // Cannot modify parameters set CA_STATIC.
+            //
+            if (  tp->flags & CA_STATIC
+               && !mudstate.bReadingConfiguration)
+            {
+                notify(player, NOPERM_MESSAGE);
+                STARTLOG(LOG_CONFIGMODS, "CFG", "PERM");
+                log_name(player);
+                log_text(" tried to change access to static param: ");
+                log_text(tp->pname);
+                ENDLOG;
+                return -1;
+            }
+            return cf_modify_bits(&tp->flags, ap, pExtra, nExtra, player, cmd);
+        }
+    }
+    cf_log_notfound(player, cmd, "Config directive", str);
+    return -1;
+}
+
 // ---------------------------------------------------------------------------
 // cf_set: Set config parameter.
 //
@@ -1937,6 +2084,11 @@ void do_admin
     char *value
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(extra);
+    UNUSED_PARAMETER(nargs);
+
     int i = cf_set(kw, value, executor);
     if ((i >= 0) && !Quiet(executor))
     {
@@ -1948,7 +2100,7 @@ void do_admin
 // ---------------------------------------------------------------------------
 // cf_read: Read in config parameters from named file
 //
-struct
+static struct
 {
     char **pFilename;
     char *pSuffix;
@@ -1984,7 +2136,7 @@ int cf_read(void)
             char *pSuffix = DefaultSuffixes[i].pSuffix;
             int nSuffix = strlen(pSuffix);
             char *buff = (char *)MEMALLOC(nInDB + nSuffix + 1);
-            (void)ISOUTOFMEMORY(buff);
+            ISOUTOFMEMORY(buff);
             memcpy(buff, mudconf.indb, nInDB);
             memcpy(buff + nInDB, pSuffix, nSuffix+1);
             MEMFREE(*p);
