@@ -1,10 +1,10 @@
 // db.cpp
 //
-// $Id: db.cpp,v 1.39 2004/09/21 06:33:03 sdennis Exp $
+// $Id: db.cpp,v 1.78 2006/01/31 00:17:26 sdennis Exp $
 //
-// MUX 2.3
-// Copyright (C) 1998 through 2003 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.  
+// MUX 2.4
+// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
+// rights not explicitly given are reserved.
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -12,6 +12,7 @@
 #include "externs.h"
 
 #include "attrs.h"
+#include "command.h"
 #include "comsys.h"
 #include "interface.h"
 #include "powers.h"
@@ -23,8 +24,6 @@
 
 OBJ *db = NULL;
 
-extern void desc_addhash(DESC *);
-
 typedef struct atrcount ATRCOUNT;
 struct atrcount
 {
@@ -32,11 +31,6 @@ struct atrcount
     int count;
 };
 
-// Check routine forward declaration.
-//
-extern void pcache_reload(dbref);
-extern void desc_reload(dbref);
-
 // list of attributes
 //
 ATTR attr[] =
@@ -121,6 +115,7 @@ ATTR attr[] =
     {"Mailsub",     A_MAILSUB,  AF_DARK | AF_NOPROG | AF_NOCMD | AF_INTERNAL},
     {"Mailsucc",    A_MAIL,     AF_ODARK | AF_NOPROG},
     {"Mailto",      A_MAILTO,   AF_DARK | AF_NOPROG | AF_NOCMD | AF_INTERNAL},
+    {"Mfail",       A_MFAIL,    AF_ODARK | AF_NOPROG},
     {"Modified",    A_MODIFIED, AF_GOD | AF_VISUAL | AF_NOPROG | AF_NOCMD},
     {"Moniker",     A_MONIKER,  AF_ODARK | AF_NOPROG | AF_NOCMD | AF_CONST},
     {"Move",        A_MOVE,     AF_ODARK | AF_NOPROG},
@@ -217,6 +212,9 @@ ATTR attr[] =
     {"*Password",   A_PASS,     AF_DARK | AF_NOPROG | AF_NOCMD | AF_INTERNAL},
     {"*Privileges", A_PRIVS,    AF_DARK | AF_NOPROG | AF_NOCMD | AF_INTERNAL},
     {"*Money",      A_MONEY,    AF_DARK | AF_NOPROG | AF_NOCMD | AF_INTERNAL},
+#ifdef REALITY_LVLS
+    {"Rlevel",         A_RLEVEL,        AF_DARK | AF_NOPROG | AF_NOCMD | AF_INTERNAL},
+#endif /* REALITY_LVLS */
     {NULL,          0,          0}
 };
 
@@ -245,7 +243,7 @@ void fwdlist_set(dbref thing, FWDLIST *ifp)
     // Copy input forwardlist to a correctly-sized buffer.
     //
     fp = (FWDLIST *)MEMALLOC(sizeof(int) * ((ifp->count) + 1));
-    (void)ISOUTOFMEMORY(fp);
+    ISOUTOFMEMORY(fp);
 
     for (i = 0; i < ifp->count; i++)
     {
@@ -390,6 +388,8 @@ int fwdlist_rewrite(FWDLIST *fp, char *atext)
  */
 bool fwdlist_ck(dbref player, dbref thing, int anum, char *atext)
 {
+    UNUSED_PARAMETER(anum);
+
     if (mudstate.bStandAlone)
     {
         return true;
@@ -679,8 +679,6 @@ void s_Pass(dbref thing, const char *s)
  * do_attrib: Manage user-named attributes.
  */
 
-extern NAMETAB attraccess_nametab[];
-
 void do_attribute
 (
     dbref executor,
@@ -692,10 +690,9 @@ void do_attribute
     char *value
 )
 {
-    int f;
-    char *sp;
-    ATTR *va2;
-    bool negate, success;
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(nargs);
 
     // Look up the user-named attribute we want to play with.
     //
@@ -703,11 +700,26 @@ void do_attribute
     bool bValid;
     ATTR *va;
     char *pName = MakeCanonicalAttributeName(aname, &nName, &bValid);
-    if (!bValid || !(va = (ATTR *)vattr_find_LEN(pName, nName)))
+    if (bValid)
+    {
+        va = (ATTR *)vattr_find_LEN(pName, nName);
+        if (!va)
+        {
+            bValid = false;
+        }
+    }
+
+    if (!bValid)
     {
         notify(executor, "No such user-named attribute.");
         return;
     }
+
+    int f;
+    char *sp;
+    ATTR *va2;
+    bool negate, success;
+
     switch (key)
     {
     case ATTRIB_ACCESS:
@@ -733,8 +745,7 @@ void do_attribute
 
             // Set or clear the appropriate bit.
             //
-            f = search_nametab(executor, attraccess_nametab, sp);
-            if (f > 0)
+            if (search_nametab(executor, attraccess_nametab, sp, &f))
             {
                 success = true;
                 if (negate)
@@ -806,6 +817,10 @@ void do_fixdb
     char *arg2
 )
 {
+    UNUSED_PARAMETER(caller);
+    UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(nargs);
+
     init_match(executor, arg1, NOTYPE);
     match_everything(0);
     dbref thing = noisy_match_result();
@@ -1036,8 +1051,9 @@ void init_attrtab(void)
     // initialization because it's part of the A_PASS attribute
     // name.
     //
-    mux_AttrNameSet['*'] = true;
-    mux_AttrNameInitialSet['*'] = true;
+    const unsigned char star = '*';
+    mux_AttrNameSet[star] = true;
+    mux_AttrNameInitialSet[star] = true;
     for (a = attr; a->number; a++)
     {
         int nLen;
@@ -1051,8 +1067,8 @@ void init_attrtab(void)
         anum_set(a->number, a);
         hashaddLEN(buff, nLen, a, &mudstate.attr_name_htab);
     }
-    mux_AttrNameInitialSet['*'] = false;
-    mux_AttrNameSet['*'] = false;
+    mux_AttrNameInitialSet[star] = false;
+    mux_AttrNameSet[star] = false;
 }
 
 /* ---------------------------------------------------------------------------
@@ -1116,7 +1132,7 @@ void anum_extend(int newtop)
     if (anum_table == NULL)
     {
         anum_table = (ATTR **) MEMALLOC((newtop + 1) * sizeof(ATTR *));
-        (void)ISOUTOFMEMORY(anum_table);
+        ISOUTOFMEMORY(anum_table);
         for (i = 0; i <= newtop; i++)
         {
             anum_table[i] = NULL;
@@ -1125,7 +1141,7 @@ void anum_extend(int newtop)
     else
     {
         anum_table2 = (ATTR **) MEMALLOC((newtop + 1) * sizeof(ATTR *));
-        (void)ISOUTOFMEMORY(anum_table2);
+        ISOUTOFMEMORY(anum_table2);
         for (i = 0; i <= anum_alc_top; i++)
         {
             anum_table2[i] = anum_table[i];
@@ -1152,11 +1168,87 @@ ATTR *atr_num(int anum)
     return  anum_get(anum);
 }
 
+static void SetupThrottle(dbref executor)
+{
+    CLinearTimeAbsolute tNow;
+    CLinearTimeDelta    ltdHour;
+
+    ltdHour.SetSeconds(60*60);
+    tNow.GetUTC();
+
+    db[executor].tThrottleExpired = tNow + ltdHour;
+    s_ThAttrib(executor, mudconf.vattr_per_hour);
+    s_ThMail(executor, mudconf.mail_per_hour);
+}
+
+static void SetupGlobalThrottle(void)
+{
+    CLinearTimeAbsolute tNow;
+    CLinearTimeDelta    ltdHour;
+
+    ltdHour.SetSeconds(60*60);
+    tNow.GetUTC();
+
+    mudstate.tThrottleExpired = tNow + ltdHour;
+    mudstate.pcreates_this_hour = mudconf.pcreate_per_hour;
+}
+
+bool ThrottlePlayerCreate(void)
+{
+    if (0 < mudstate.pcreates_this_hour)
+    {
+        mudstate.pcreates_this_hour--;
+        return false;
+    }
+    CLinearTimeAbsolute tNow;
+    tNow.GetUTC();
+    if (mudstate.tThrottleExpired <= tNow)
+    {
+        SetupGlobalThrottle();
+        return false;
+    }
+    return true;
+}
+
+bool ThrottleAttributeNames(dbref executor)
+{
+    if (0 < ThAttrib(executor))
+    {
+        s_ThAttrib(executor, ThAttrib(executor)-1);
+        return false;
+    }
+    CLinearTimeAbsolute tNow;
+    tNow.GetUTC();
+    if (db[executor].tThrottleExpired <= tNow)
+    {
+        SetupThrottle(executor);
+        return false;
+    }
+    return true;
+}
+
+bool ThrottleMail(dbref executor)
+{
+    if (0 < ThMail(executor))
+    {
+        s_ThMail(executor, ThMail(executor)-1);
+        return false;
+    }
+    CLinearTimeAbsolute tNow;
+    tNow.GetUTC();
+    if (db[executor].tThrottleExpired <= tNow)
+    {
+        SetupThrottle(executor);
+        return false;
+    }
+    return true;
+}
+
 /* ---------------------------------------------------------------------------
  * mkattr: Lookup attribute by name, creating if needed.
  */
 
-int mkattr(char *buff)
+int mkattr(dbref executor, char *buff)
 {
     ATTR *ap = atr_str(buff);
     if (!ap)
@@ -1169,6 +1261,11 @@ int mkattr(char *buff)
         ATTR *va;
         if (bValid)
         {
+            if (  !Wizard(executor)
+               && ThrottleAttributeNames(executor))
+            {
+                return -1;
+            }
             int aflags = mudconf.vattr_flags;
             if (pName[0] == '_')
             {
@@ -1235,12 +1332,15 @@ static char *al_code(char *ap, int atrnum)
     for (;;)
     {
         bits = atrnum & 0x7F;
-        if (atrnum <= 0x7F) break;
+        if (atrnum <= 0x7F)
+        {
+            break;
+        }
         atrnum >>= 7;
         bits |= 0x80;
-        *ap++ = bits;
+        *ap++ = (char)bits;
     }
-    *ap++ = bits;
+    *ap++ = (char)bits;
     return ap;
 }
 
@@ -1250,28 +1350,87 @@ static char *al_code(char *ap, int atrnum)
 
 bool Commer(dbref thing)
 {
-    char *s, *as, c;
-    int attr, aflags;
-    dbref aowner;
-    ATTR *ap;
-
-    atr_push();
-    for (attr = atr_head(thing, &as); attr; attr = atr_next(&as))
+    if (mudstate.bfNoCommands.IsSet(thing))
     {
-        ap = atr_num(attr);
-        if (!ap || (ap->flags & AF_NOPROG))
-            continue;
+        // We already know that there are no commands on this thing.
+        //
+        return false;
+    }
+    else if (mudstate.bfCommands.IsSet(thing))
+    {
+        // We already know that there are definitely commands on this thing.
+        //
+        return true;
+    }
 
-        s = atr_get(thing, attr, &aowner, &aflags);
-        c = *s;
-        free_lbuf(s);
-        if ((c == '$') && !(aflags & AF_NOPROG))
+    bool bFoundListens = false;
+
+    char *as;
+    atr_push();
+    char *buff = alloc_lbuf("Commer");
+    for (int atr = atr_head(thing, &as); atr; atr = atr_next(&as))
+    {
+        ATTR *ap = atr_num(atr);
+        if (  !ap
+           || (ap->flags & AF_NOPROG))
         {
+            continue;
+        }
+
+        int   aflags;
+        dbref aowner;
+
+        atr_get_str(buff, thing, atr, &aowner, &aflags);
+
+        if (aflags & AF_NOPROG)
+        {
+            continue;
+        }
+
+        if (  AMATCH_CMD != buff[0]
+           && AMATCH_LISTEN != buff[0])
+        {
+            continue;
+        }
+
+        // Search for unescaped ':'
+        //
+        char *s = strchr(buff+1, ':');
+        if (!s)
+        {
+            continue;
+        }
+
+        if (AMATCH_CMD == buff[0])
+        {
+            free_lbuf(buff);
             atr_pop();
+            mudstate.bfCommands.Set(thing);
+            if (bFoundListens)
+            {
+                mudstate.bfListens.Set(thing);
+                mudstate.bfNoListens.Clear(thing);
+            }
             return true;
         }
+        else // AMATCH_LISTEN == buff[0]
+        {
+            bFoundListens = true;
+        }
     }
+    free_lbuf(buff);
     atr_pop();
+    mudstate.bfNoCommands.Set(thing);
+    if (bFoundListens)
+    {
+        mudstate.bfListens.Set(thing);
+        mudstate.bfNoListens.Clear(thing);
+    }
+    else
+    {
+        mudstate.bfNoListens.Set(thing);
+        mudstate.bfListens.Clear(thing);
+    }
     return false;
 }
 
@@ -1285,13 +1444,13 @@ bool Commer(dbref thing)
 
 // al_extend: Get more space for attributes, if needed
 //
-void al_extend(char **buffer, size_t *bufsiz, size_t len, bool copy)
+static void al_extend(char **buffer, size_t *bufsiz, size_t len, bool copy)
 {
     if (len > *bufsiz)
     {
         int newsize = len + ATR_BUF_CHUNK;
         char *tbuff = (char *)MEMALLOC(newsize);
-        (void)ISOUTOFMEMORY(tbuff);
+        ISOUTOFMEMORY(tbuff);
         if (*buffer)
         {
             if (copy)
@@ -1326,7 +1485,7 @@ void al_store(void)
 
 // al_fetch: Load attribute list
 //
-char *al_fetch(dbref thing)
+static char *al_fetch(dbref thing)
 {
     // We only need fetch if we change things.
     //
@@ -1358,7 +1517,7 @@ char *al_fetch(dbref thing)
 
 // al_add: Add an attribute to an attribute list
 //
-bool al_add(dbref thing, int attrnum)
+static bool al_add(dbref thing, int attrnum)
 {
     char *abuf = al_fetch(thing);
     char *cp = abuf;
@@ -1406,7 +1565,7 @@ bool al_add(dbref thing, int attrnum)
 
 // al_delete: Remove an attribute from an attribute list
 //
-void al_delete(dbref thing, int attrnum)
+static void al_delete(dbref thing, int attrnum)
 {
     int anum;
     char *abuf, *cp, *dp;
@@ -1443,7 +1602,7 @@ void al_delete(dbref thing, int attrnum)
     return;
 }
 
-DCL_INLINE static void makekey(dbref thing, int atr, Aname *abuff)
+static DCL_INLINE void makekey(dbref thing, int atr, Aname *abuff)
 {
     abuff->object = thing;
     abuff->attrnum = atr;
@@ -1457,6 +1616,7 @@ DCL_INLINE static void makekey(dbref thing, int atr, Aname *abuff)
 
 static char *atr_encode(char *iattr, dbref thing, dbref owner, int flags, int atr)
 {
+    UNUSED_PARAMETER(atr);
 
     // If using the default owner and flags (almost all attributes will),
     // just store the string.
@@ -1618,6 +1778,7 @@ void atr_clr(dbref thing, int atr)
     cache_del(&okey);
     al_delete(thing, atr);
 #endif // MEMORY_BASED
+
     switch (atr)
     {
     case A_STARTUP:
@@ -1655,6 +1816,15 @@ void atr_clr(dbref thing, int atr)
 
         pcache_reload(thing);
         break;
+
+    default:
+
+        // Since this could overwrite an existing ^-Command or $-Command, we
+        // longer assert that the object has one.
+        //
+        mudstate.bfListens.Clear(thing);
+        mudstate.bfCommands.Clear(thing);
+        break;
     }
 }
 
@@ -1664,18 +1834,13 @@ void atr_clr(dbref thing, int atr)
 
 void atr_add_raw_LEN(dbref thing, int atr, const char *szValue, int nValue)
 {
-#ifdef MEMORY_BASED
-
     if (!szValue || szValue[0] == '\0')
     {
         atr_clr(thing, atr);
         return;
     }
 
-    if (nValue > LBUF_SIZE-1)
-    {
-        nValue = LBUF_SIZE-1;
-    }
+#ifdef MEMORY_BASED
     ATRLIST *list;
     bool found = false;
     int hi, lo, mid;
@@ -1684,7 +1849,7 @@ void atr_add_raw_LEN(dbref thing, int atr, const char *szValue, int nValue)
     if (!db[thing].ahead)
     {
         list = (ATRLIST *)MEMALLOC(sizeof(ATRLIST));
-        (void)ISOUTOFMEMORY(list);
+        ISOUTOFMEMORY(list);
         db[thing].ahead = list;
         db[thing].at_count = 1;
         list[0].number = atr;
@@ -1728,7 +1893,7 @@ void atr_add_raw_LEN(dbref thing, int atr, const char *szValue, int nValue)
             // and the attribute should be inserted between them.
             //
             list = (ATRLIST *)MEMALLOC((db[thing].at_count + 1) * sizeof(ATRLIST));
-            (void)ISOUTOFMEMORY(list);
+            ISOUTOFMEMORY(list);
 
             // Copy bottom part.
             //
@@ -1753,22 +1918,16 @@ void atr_add_raw_LEN(dbref thing, int atr, const char *szValue, int nValue)
             db[thing].ahead = list;
         }
     }
-#else // MEMORY_BASED
-    Aname okey;
 
-    makekey(thing, atr, &okey);
-    if (!szValue || szValue[0] == '\0')
-    {
-        cache_del(&okey);
-        al_delete(thing, atr);
-        return;
-    }
+#else // MEMORY_BASED
 
     if (nValue > LBUF_SIZE-1)
     {
         nValue = LBUF_SIZE-1;
     }
 
+    Aname okey;
+    makekey(thing, atr, &okey);
     if (atr == A_LIST)
     {
         // A_LIST is never compressed and it's never listed within itself.
@@ -1834,6 +1993,31 @@ void atr_add(dbref thing, int atr, char *buff, dbref owner, int flags)
     }
     else
     {
+        switch (buff[0])
+        {
+        case AMATCH_LISTEN:
+
+            // Since this could be a ^-Command, we no longer assert that the
+            // object has none.
+            //
+            mudstate.bfNoListens.Clear(thing);
+            break;
+
+        case AMATCH_CMD:
+
+            // Since this could be a $-Command, we no longer assert that the
+            // object has none.
+            //
+            mudstate.bfNoCommands.Clear(thing);
+            break;
+        }
+
+        // Since this could overwrite an existing ^-Command or $-Command, we
+        // longer assert that the object has one.
+        //
+        mudstate.bfListens.Clear(thing);
+        mudstate.bfCommands.Clear(thing);
+
         char *tbuff = atr_encode(buff, thing, owner, flags, atr);
         atr_add_raw(thing, atr, tbuff);
     }
@@ -2076,58 +2260,67 @@ void atr_free(dbref thing)
     db[thing].ahead = NULL;
     db[thing].at_count = 0;
 #else // MEMORY_BASED
-    int attr;
     char *as;
     atr_push();
-    for (attr = atr_head(thing, &as); attr; attr = atr_next(&as))
+    for (int atr = atr_head(thing, &as); atr; atr = atr_next(&as))
     {
-        atr_clr(thing, attr);
+        atr_clr(thing, atr);
     }
     atr_pop();
     if (mudstate.mod_al_id == thing)
+    {
         al_store(); // remove from cache
+    }
     atr_clr(thing, A_LIST);
 #endif // MEMORY_BASED
+
+    mudstate.bfCommands.Clear(thing);
+    mudstate.bfNoCommands.Set(thing);
+    mudstate.bfListens.Clear(thing);
+    mudstate.bfNoListens.Set(thing);
 }
 
 /* ---------------------------------------------------------------------------
- * atr_cpy: Copy all attributes from one object to another.  
+ * atr_cpy: Copy all attributes from one object to another.
  */
 
-void atr_cpy(dbref dest, dbref source)
+void atr_cpy(dbref dest, dbref source, bool bInternal)
 {
-    int attr, aflags;
-    dbref aowner;
-    char *as, *buf;
-    ATTR *at;
-
     dbref owner = Owner(dest);
+
+    char *as;
     atr_push();
-    for (attr = atr_head(source, &as); attr; attr = atr_next(&as))
+    for (int atr = atr_head(source, &as); atr; atr = atr_next(&as))
     {
-        buf = atr_get(source, attr, &aowner, &aflags);
+        int   aflags;
+        dbref aowner;
+        char *buf = atr_get(source, atr, &aowner, &aflags);
+
         if (!(aflags & AF_LOCK))
         {
             // Change owner.
             //
             aowner = owner;
         }
-        at = atr_num(attr);
-        if (attr && at)
+
+        ATTR *at = atr_num(atr);
+        if (  atr
+           && at)
         {
             if (  !(at->flags & (AF_INTERNAL|AF_NOCLONE))
-               && (  God(owner) 
-                  || (  !God(dest) 
-                     && !(aflags & AF_LOCK) 
-                     && (  (  Controls(owner, dest) 
-                           && !(at->flags & (AF_WIZARD|AF_GOD)) 
+               && (  bInternal
+                  || God(owner)
+                  || (  !God(dest)
+                     && !(aflags & AF_LOCK)
+                     && (  (  Controls(owner, dest)
+                           && !(at->flags & (AF_WIZARD|AF_GOD))
                            && !(aflags & (AF_WIZARD|AF_GOD)))
                         || (  Wizard(owner)
                            && !(at->flags & AF_GOD))))))
             {
                 // Only set attrs that owner has perm to set.
                 //
-                atr_add(dest, attr, buf, aowner, aflags);
+                atr_add(dest, atr, buf, aowner, aflags);
             }
         }
         free_lbuf(buf);
@@ -2142,17 +2335,20 @@ void atr_cpy(dbref dest, dbref source)
 
 void atr_chown(dbref obj)
 {
-    int attr, aflags;
-    dbref aowner;
-    char *as, *buf;
-
     dbref owner = Owner(obj);
+
+    char *as;
     atr_push();
-    for (attr = atr_head(obj, &as); attr; attr = atr_next(&as))
+    for (int atr = atr_head(obj, &as); atr; atr = atr_next(&as))
     {
-        buf = atr_get(obj, attr, &aowner, &aflags);
-        if ((aowner != owner) && !(aflags & AF_LOCK))
-            atr_add(obj, attr, buf, owner, aflags);
+        int   aflags;
+        dbref aowner;
+        char *buf = atr_get(obj, atr, &aowner, &aflags);
+        if (  aowner != owner
+           && !(aflags & AF_LOCK))
+        {
+            atr_add(obj, atr, buf, owner, aflags);
+        }
         free_lbuf(buf);
     }
     atr_pop();
@@ -2251,7 +2447,7 @@ int atr_head(dbref thing, char **attrp)
     if (db[thing].at_count)
     {
         ATRCOUNT *atr = (ATRCOUNT *) MEMALLOC(sizeof(ATRCOUNT));
-        (void)ISOUTOFMEMORY(atr);
+        ISOUTOFMEMORY(atr);
         atr->thing = thing;
         atr->count = 1;
         *attrp = (char *)atr;
@@ -2297,7 +2493,7 @@ int atr_head(dbref thing, char **attrp)
 
 #define SIZE_HACK   1   // So mistaken refs to #-1 won't die.
 
-void initialize_objects(dbref first, dbref last)
+static void initialize_objects(dbref first, dbref last)
 {
     dbref thing;
 
@@ -2316,6 +2512,10 @@ void initialize_objects(dbref first, dbref last)
         s_Parent(thing, NOTHING);
         s_Stack(thing, NULL);
         db[thing].cpu_time_used.Set100ns(0);
+        db[thing].tThrottleExpired.Set100ns(0);
+        s_ThAttrib(thing, 0);
+        s_ThMail(thing, 0);
+
 #ifdef MEMORY_BASED
         db[thing].ahead = NULL;
         db[thing].at_count = 0;
@@ -2329,6 +2529,11 @@ void initialize_objects(dbref first, dbref last)
 
 void db_grow(dbref newtop)
 {
+    mudstate.bfCommands.Resize(newtop);
+    mudstate.bfNoCommands.Resize(newtop);
+    mudstate.bfListens.Resize(newtop);
+    mudstate.bfNoListens.Resize(newtop);
+
     int delta;
     if (mudstate.bStandAlone)
     {
@@ -2388,7 +2593,7 @@ void db_grow(dbref newtop)
     // to reclaim the memory.
     //
     OBJ *newdb = (OBJ *)MEMALLOC((newsize + SIZE_HACK) * sizeof(OBJ));
-    (void)ISOUTOFMEMORY(newdb);
+    ISOUTOFMEMORY(newdb);
     if (db)
     {
         // An old struct database exists. Copy it to the new buffer.
@@ -2416,7 +2621,7 @@ void db_grow(dbref newtop)
     //
     int marksize = (newsize + 7) >> 3;
     MARKBUF *newmarkbuf = (MARKBUF *)MEMALLOC(marksize);
-    (void)ISOUTOFMEMORY(newmarkbuf);
+    ISOUTOFMEMORY(newmarkbuf);
     memset(newmarkbuf, 0, marksize);
     if (mudstate.markbits)
     {
@@ -2482,16 +2687,23 @@ void db_make_minimal(void)
 
 dbref parse_dbref(const char *s)
 {
-    const char *p;
-
     // Enforce completely numeric dbrefs
     //
-    for (p = s; *p; p++)
+    const char *p = s;
+    if (p[0])
     {
-        if (!mux_isdigit(*p))
+        do
         {
-            return NOTHING;
-        }
+            if (!mux_isdigit(*p))
+            {
+                return NOTHING;
+            }
+            p++;
+        } while (*p);
+    }
+    else
+    {
+        return NOTHING;
     }
     int x = mux_atol(s);
     return ((x >= 0) ? x : NOTHING);
@@ -2507,11 +2719,11 @@ void putref(FILE *f, dbref ref)
 }
 
 // Code 0 - Any byte.
-// Code 1 - CR (0x0D)
-// Code 2 - '"' (0x22)
+// Code 1 - NUL  (0x00)
+// Code 2 - '"'  (0x22)
 // Code 3 - '\\' (0x5C)
 //
-static const char xlat_table[256] =
+static const unsigned char xlat_table[256] =
 {
     1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -2754,7 +2966,7 @@ void free_boolexp(BOOLEXP *b)
     }
 }
 
-BOOLEXP *dup_bool(BOOLEXP *b)
+static BOOLEXP *dup_bool(BOOLEXP *b)
 {
     if (b == TRUE_BOOLEXP)
         return (TRUE_BOOLEXP);
@@ -2874,7 +3086,7 @@ void dump_restart_db(void)
 {
     FILE *f;
     DESC *d;
-    int version = 1;
+    int version = 2;
 
     f = fopen("restart.db", "wb");
     fprintf(f, "+V%d\n", version);
@@ -2897,6 +3109,15 @@ void dump_restart_db(void)
         putref(f, d->host_info);
         putref(f, d->player);
         putref(f, d->last_time.ReturnSeconds());
+        putref(f, d->raw_input_state);
+        putref(f, d->nvt_sga_him_state);
+        putref(f, d->nvt_sga_us_state);
+        putref(f, d->nvt_eor_him_state);
+        putref(f, d->nvt_eor_us_state);
+        putref(f, d->nvt_naws_him_state);
+        putref(f, d->nvt_naws_us_state);
+        putref(f, d->height);
+        putref(f, d->width);
         putstring(f, d->output_prefix);
         putstring(f, d->output_suffix);
         putstring(f, d->addr);
@@ -2928,9 +3149,11 @@ void load_restart_db(void)
     fgets(buf, 3, f);
     mux_assert(strncmp(buf, "+V", 2) == 0);
     int version = getref(f);
-    if (version == 1)
+    if (  1 == version
+       || 2 == version)
     {
-        // Started on 2001-DEC-03
+        // Version 1 started on 2001-DEC-03
+        // Version 2 started on 2005-NOV-08
         //
         nMainGamePorts = getref(f);
         for (int i = 0; i < nMainGamePorts; i++)
@@ -2957,6 +3180,10 @@ void load_restart_db(void)
     mudstate.start_time.SetSeconds(getref(f));
     strcpy(mudstate.doing_hdr, getstring_noalloc(f, true));
     mudstate.record_players = getref(f);
+    if (mudconf.reset_players)
+    {
+        mudstate.record_players = 0;
+    }
 
     while ((val = getref(f)) != 0)
     {
@@ -2971,6 +3198,31 @@ void load_restart_db(void)
         d->host_info = getref(f);
         d->player = getref(f);
         d->last_time.SetSeconds(getref(f));
+        if (2 == version)
+        {
+            d->raw_input_state    = getref(f);
+            d->nvt_sga_him_state  = getref(f);
+            d->nvt_sga_us_state   = getref(f);
+            d->nvt_eor_him_state  = getref(f);
+            d->nvt_eor_us_state   = getref(f);
+            d->nvt_naws_him_state = getref(f);
+            d->nvt_naws_us_state  = getref(f);
+            d->height = getref(f);
+            d->width = getref(f);
+        }
+        else
+        {
+            d->raw_input_state    = NVT_IS_NORMAL;
+            d->nvt_sga_him_state  = OPTION_NO;
+            d->nvt_sga_us_state   = OPTION_NO;
+            d->nvt_eor_him_state  = OPTION_NO;
+            d->nvt_eor_us_state   = OPTION_NO;
+            d->nvt_naws_him_state = OPTION_NO;
+            d->nvt_naws_us_state  = OPTION_NO;
+            d->height = 24;
+            d->width = 78;
+        }
+
         temp = getstring_noalloc(f, true);
         if (*temp)
         {
@@ -3008,6 +3260,7 @@ void load_restart_db(void)
         d->input_lost = 0;
         d->raw_input = NULL;
         d->raw_input_at = NULL;
+        d->nOption = 0;
         d->quota = mudconf.cmd_quota_max;
         d->program_data = NULL;
         d->hashnext = NULL;
