@@ -1,6 +1,6 @@
 // db_rw.cpp
 //
-// $Id: db_rw.cpp,v 1.12 2003/02/17 02:34:21 sdennis Exp $
+// $Id: db_rw.cpp,v 1.22 2006/01/07 07:31:06 sdennis Exp $
 //
 
 #include "copyright.h"
@@ -11,10 +11,6 @@
 #include "attrs.h"
 #include "vattr.h"
 
-extern void db_grow(dbref);
-
-extern struct object *db;
-
 static int g_version;
 static int g_format;
 static int g_flags;
@@ -154,7 +150,6 @@ static BOOLEXP *getboolexp1(FILE *f)
         }
         ungetc(c, f);
         return TRUE_BOOLEXP;
-        break;
 
     case '"':
         ungetc(c, f);
@@ -168,7 +163,7 @@ static BOOLEXP *getboolexp1(FILE *f)
         }
 
         b = alloc_bool("getboolexp1_quoted");
-        anum = mkattr(buff);
+        anum = mkattr(GOD, buff);
         if (anum <= 0)
         {
             free_bool(b);
@@ -227,9 +222,10 @@ static BOOLEXP *getboolexp1(FILE *f)
                    (c = getc(f)) != EOF
                 && c != '\n'
                 && c != ':'
-                && c != '/';
+                && c != '/'
+                && s < buff + LBUF_SIZE;
 
-                   *s++ = c)
+                   *s++ = (char)c)
             {
                 ; // Nothing.
             }
@@ -245,7 +241,7 @@ static BOOLEXP *getboolexp1(FILE *f)
             // Look the name up as an attribute. If not found, create a new
             // attribute.
             //
-            anum = mkattr(buff);
+            anum = mkattr(GOD, buff);
             if (anum <= 0)
             {
                 free_bool(b);
@@ -282,9 +278,10 @@ static BOOLEXP *getboolexp1(FILE *f)
                 && c != '\n'
                 && c != ')'
                 && c != OR_TOKEN
-                && c != AND_TOKEN;
+                && c != AND_TOKEN
+                && s < buff + LBUF_SIZE;
 
-                   *s++ = c)
+                   *s++ = (char)c)
             {
                 ; // Nothing
             }
@@ -315,7 +312,7 @@ error:
 static BOOLEXP *getboolexp(FILE *f)
 {
     BOOLEXP *b = getboolexp1(f);
-    char c = getc(f);
+    int c = getc(f);
     mux_assert(c == '\n');
 
     if (g_format == F_MUX)
@@ -335,7 +332,7 @@ static BOOLEXP *getboolexp(FILE *f)
 static bool get_list(FILE *f, dbref i)
 {
     char *buff = alloc_lbuf("get_list");
-    while (1)
+    for (;;)
     {
         dbref atr;
         int c;
@@ -509,7 +506,6 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
     int aflags;
     BOOLEXP *tempbool;
     char *buff;
-    int len;
     int nVisualWidth;
 
     g_format = F_UNKNOWN;
@@ -559,6 +555,10 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
                 // Record number of players
                 //
                 mudstate.record_players = getref(f);
+                if (mudconf.reset_players)
+                {
+                    mudstate.record_players = 0;
+                }
             }
             break;
 
@@ -667,7 +667,7 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
             {
                 tstr = getstring_noalloc(f, true);
                 buff = alloc_lbuf("dbread.s_Name");
-                len = ANSI_TruncateToField(tstr, MBUF_SIZE, buff, MBUF_SIZE,
+                (void)ANSI_TruncateToField(tstr, MBUF_SIZE, buff, MBUF_SIZE,
                     &nVisualWidth, ANSI_ENDGOAL_NORMAL);
                 s_Name(i, buff);
                 free_lbuf(buff);
@@ -801,6 +801,8 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
 
 static bool db_write_object(FILE *f, dbref i, int db_format, int flags)
 {
+    UNUSED_PARAMETER(db_format);
+
     ATTR *a;
     char *got, *as;
     dbref aowner;
@@ -897,18 +899,12 @@ static bool db_write_object(FILE *f, dbref i, int db_format, int flags)
     return false;
 }
 
-extern int anum_alc_top;
-
 dbref db_write(FILE *f, int format, int version)
 {
     dbref i;
     int flags;
     ATTR *vp;
 
-#ifndef MEMORY_BASED
-    al_store();
-#endif
-
     switch (format)
     {
     case F_MUX:
