@@ -1,10 +1,10 @@
 // eval.cpp -- Command evaluation and cracking.
 //
-// $Id: eval.cpp,v 1.22 2003/03/13 15:59:55 jake Exp $
+// $Id: eval.cpp,v 1.33 2006/01/19 18:21:47 sdennis Exp $
 //
-// MUX 2.3
-// Copyright (C) 1998 through 2003 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.  
+// MUX 2.4
+// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
+// rights not explicitly given are reserved.
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -69,6 +69,14 @@ static char *parse_to_cleanup( int eval, int first, char *cstr, char *rstr,
     return rstr;
 }
 
+/*! \brief Accesses the isSpecial_ tables with proper typecast.
+ *
+ * \param table    indicates which table: \c L1, \c L2, \c L3, or \c L4.
+ * \param c        character being looked up.
+ * \return         lvalue of table entry.
+ */
+#define isSpecial(table, c) isSpecial_##table[(unsigned char)(c)]
+
 // During parsing, this table may be modified for a particular terminating delimeter.
 // The table is always restored it's original state.
 //
@@ -201,18 +209,18 @@ char *parse_to(char **dstr, char delim, int eval)
         *dstr = rstr;
     }
     zstr = cstr = rstr;
-    int iOriginalCode = isSpecial_L3[(unsigned char)delim];
-    isSpecial_L3[' '] = 1; // Spaces are special.
+    int iOriginalCode = isSpecial(L3, delim);
+    isSpecial(L3, ' ') = 1; // Spaces are special.
     if (iOriginalCode <= 3)
     {
         // We can override this code.
         //
-        isSpecial_L3[(unsigned char)delim] = 8;
+        isSpecial(L3, delim) = 8;
     }
 
     for (;;)
     {
-        int iCode = isSpecial_L3[(unsigned char)*cstr];
+        int iCode = isSpecial(L3, *cstr);
 
 TryAgain:
         if (iCode == 0)
@@ -223,7 +231,7 @@ TryAgain:
             do
             {
                 NEXTCHAR
-                iCode = isSpecial_L3[(unsigned char)*cstr];
+                iCode = isSpecial(L3, *cstr);
             } while (iCode == 0);
         }
 
@@ -331,8 +339,8 @@ TryAgain:
                     {
                         rstr = parse_to_cleanup(eval, first, cstr, rstr, zstr, strFirewall);
                         *dstr = ++cstr;
-                        isSpecial_L3[(unsigned char)delim] = iOriginalCode;
-                        isSpecial_L3[' '] = 0; // Spaces aren't special anymore
+                        isSpecial(L3, delim) = iOriginalCode;
+                        isSpecial(L3, ' ') = 0; // Spaces no longer special
                         return rstr;
                     }
                     first = false;
@@ -353,7 +361,7 @@ TryAgain:
                     }
                     for (;;)
                     {
-                        int iCodeL4 = isSpecial_L4[(unsigned char)*cstr];
+                        int iCodeL4 = isSpecial(L4, *cstr);
                         if (iCodeL4 == 0)
                         {
                             // Mudane Characters
@@ -361,7 +369,7 @@ TryAgain:
                             do
                             {
                                 NEXTCHAR
-                                iCodeL4 = isSpecial_L4[(unsigned char)*cstr];
+                                iCodeL4 = isSpecial(L4, *cstr);
                             } while (iCodeL4 == 0);
                         }
 
@@ -423,8 +431,8 @@ TryAgain:
                 {
                     // '\0' - End of string.
                     //
-                    isSpecial_L3[(unsigned char)delim] = iOriginalCode;
-                    isSpecial_L3[' '] = 0; // Spaces aren't special anymore
+                    isSpecial(L3, delim) = iOriginalCode;
+                    isSpecial(L3, ' ') = 0; // Spaces no longer special
                     break;
                 }
                 else
@@ -435,8 +443,8 @@ TryAgain:
                     {
                         rstr = parse_to_cleanup(eval, first, cstr, rstr, zstr, strFirewall);
                         *dstr = ++cstr;
-                        isSpecial_L3[(unsigned char)delim] = iOriginalCode;
-                        isSpecial_L3[' '] = 0; // Spaces aren't special anymore
+                        isSpecial(L3, delim) = iOriginalCode;
+                        isSpecial(L3, ' ') = 0; // Spaces no longer special
                         return rstr;
                     }
 
@@ -458,7 +466,7 @@ TryAgain:
 // the characters to another buffer anyway and is more than able to perform the
 // escapes and trimming.
 //
-char *parse_to_lite(char **dstr, char delim1, char delim2, int *nLen, int *iWhichDelim)
+static char *parse_to_lite(char **dstr, char delim1, char delim2, int *nLen, int *iWhichDelim)
 {
 #define stacklim 32
     char stack[stacklim];
@@ -481,24 +489,24 @@ char *parse_to_lite(char **dstr, char delim1, char delim2, int *nLen, int *iWhic
     }
     sp = 0;
     cstr = rstr = *dstr;
-    int iOriginalCode1 = isSpecial_L3[(unsigned char)delim1];
-    int iOriginalCode2 = isSpecial_L3[(unsigned char)delim2];
+    int iOriginalCode1 = isSpecial(L3, delim1);
+    int iOriginalCode2 = isSpecial(L3, delim2);
     if (iOriginalCode1 <= 3)
     {
         // We can override this code.
         //
-        isSpecial_L3[(unsigned char)delim1] = 8;
+        isSpecial(L3, delim1) = 8;
     }
     if (iOriginalCode2 <= 3)
     {
         // We can override this code.
         //
-        isSpecial_L3[(unsigned char)delim2] = 8;
+        isSpecial(L3, delim2) = 8;
     }
 
     for (;;)
     {
-        int iCode = isSpecial_L3[(unsigned char)*cstr];
+        int iCode = isSpecial(L3, *cstr);
 
 TryAgain:
         if (iCode == 0)
@@ -508,7 +516,7 @@ TryAgain:
             do
             {
                 cstr++;
-                iCode = isSpecial_L3[(unsigned char)*cstr];
+                iCode = isSpecial(L3, *cstr);
             } while (iCode == 0);
         }
 
@@ -585,8 +593,8 @@ TryAgain:
                         *cstr = '\0';
                         *nLen = (cstr - rstr);
                         *dstr = ++cstr;
-                        isSpecial_L3[(unsigned char)delim1] = iOriginalCode1;
-                        isSpecial_L3[(unsigned char)delim2] = iOriginalCode2;
+                        isSpecial(L3, delim1) = iOriginalCode1;
+                        isSpecial(L3, delim2) = iOriginalCode2;
                         return rstr;
                     }
                     cstr++;
@@ -599,7 +607,7 @@ TryAgain:
                     cstr++;
                     for (;;)
                     {
-                        int iCodeL4 = isSpecial_L4[(unsigned char)*cstr];
+                        int iCodeL4 = isSpecial(L4, *cstr);
                         if (iCodeL4 == 0)
                         {
                             // Mudane Characters
@@ -607,7 +615,7 @@ TryAgain:
                             do
                             {
                                 cstr++;
-                                iCodeL4 = isSpecial_L4[(unsigned char)*cstr];
+                                iCodeL4 = isSpecial(L4, *cstr);
                             } while (iCodeL4 == 0);
                         }
 
@@ -661,8 +669,8 @@ TryAgain:
                 {
                     // '\0' - End of string.
                     //
-                    isSpecial_L3[(unsigned char)delim1] = iOriginalCode1;
-                    isSpecial_L3[(unsigned char)delim2] = iOriginalCode2;
+                    isSpecial(L3, delim1) = iOriginalCode1;
+                    isSpecial(L3, delim2) = iOriginalCode2;
                     break;
                 }
                 else
@@ -682,8 +690,8 @@ TryAgain:
                         *cstr = '\0';
                         *nLen = (cstr - rstr);
                         *dstr = ++cstr;
-                        isSpecial_L3[(unsigned char)delim1] = iOriginalCode1;
-                        isSpecial_L3[(unsigned char)delim2] = iOriginalCode2;
+                        isSpecial(L3, delim1) = iOriginalCode1;
+                        isSpecial(L3, delim2) = iOriginalCode2;
                         return rstr;
                     }
 
@@ -765,11 +773,13 @@ char *parse_arglist( dbref executor, dbref caller, dbref enactor, char *dstr,
     return dstr;
 }
 
-char *parse_arglist_lite( dbref executor, dbref caller, dbref enactor,
+static char *parse_arglist_lite( dbref executor, dbref caller, dbref enactor,
                           char *dstr, char delim, int eval, char *fargs[],
                           dbref nfargs, char *cargs[], dbref ncargs,
                           int *nArgsParsed)
 {
+    UNUSED_PARAMETER(delim);
+
     char *tstr, *bp, *str;
 
     if (dstr == NULL)
@@ -850,7 +860,7 @@ int get_gender(dbref player)
 // Trace cache routines.
 //
 typedef struct tcache_ent TCENT;
-struct tcache_ent
+static struct tcache_ent
 {
     dbref player;
     char *orig;
@@ -858,8 +868,8 @@ struct tcache_ent
     struct tcache_ent *next;
 } *tcache_head;
 
-bool tcache_top;
-int  tcache_count;
+static bool tcache_top;
+static int  tcache_count;
 
 void tcache_init(void)
 {
@@ -868,7 +878,7 @@ void tcache_init(void)
     tcache_count = 0;
 }
 
-bool tcache_empty(void)
+static bool tcache_empty(void)
 {
     if (tcache_top)
     {
@@ -1024,11 +1034,13 @@ char **PushPointers(int nNeeded)
 
 void PopPointers(char **p, int nNeeded)
 {
+    UNUSED_PARAMETER(p);
+
     if (pPtrsFrame->nptrs == PTRS_PER_FRAME)
     {
-        PtrsFrame *p = pPtrsFrame->next;
+        PtrsFrame *q = pPtrsFrame->next;
         free_lbuf((char *)pPtrsFrame);
-        pPtrsFrame = p;
+        pPtrsFrame = q;
     }
     //mux_assert(p == pPtrsFrame->ptrs + pPtrsFrame->nptrs);
     pPtrsFrame->nptrs += nNeeded;
@@ -1060,6 +1072,8 @@ int *PushIntegers(int nNeeded)
 
 void PopIntegers(int *pi, int nNeeded)
 {
+    UNUSED_PARAMETER(pi);
+
     if (pIntsFrame->nints == INTS_PER_FRAME)
     {
         IntsFrame *p = pIntsFrame->next;
@@ -1074,7 +1088,8 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                dbref enactor, int eval, char **dstr, char *cargs[], int ncargs)
 {
     if (  *dstr == NULL
-       || **dstr == '\0')
+       || **dstr == '\0'
+       || MuxAlarm.bAlarmed)
     {
         return;
     }
@@ -1124,7 +1139,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
         realbuff = buff;
         realbp = *bufc;
         buff = (char *)MEMALLOC(LBUF_SIZE);
-        (void)ISOUTOFMEMORY(buff);
+        ISOUTOFMEMORY(buff);
         *bufc = buff;
     }
 
@@ -1142,16 +1157,16 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
 
     // Save Parser Mode.
     //
-    bool bSpaceIsSpecialSave = isSpecial_L1[' '];
-    bool bParenthesisIsSpecialSave = isSpecial_L1['('];
-    bool bBracketIsSpecialSave = isSpecial_L1['['];
+    bool bSpaceIsSpecialSave = isSpecial(L1, ' ');
+    bool bParenthesisIsSpecialSave = isSpecial(L1, '(');
+    bool bBracketIsSpecialSave = isSpecial(L1, '[');
 
     // Setup New Parser Mode.
     //
     bool bSpaceIsSpecial = mudconf.space_compress && !(eval & EV_NO_COMPRESS);
-    isSpecial_L1[' '] = bSpaceIsSpecial;
-    isSpecial_L1['('] = (eval & EV_FCHECK) != 0;
-    isSpecial_L1['['] = (eval & EV_NOFCHECK) == 0;
+    isSpecial(L1, ' ') = bSpaceIsSpecial;
+    isSpecial(L1, '(') = (eval & EV_FCHECK) != 0;
+    isSpecial(L1, '[') = (eval & EV_NOFCHECK) == 0;
 
     size_t nBufferAvailable = LBUF_SIZE - (*bufc - buff) - 1;
     for (;;)
@@ -1159,10 +1174,10 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
         // Handle mundane characters specially. There are usually a lot of them.
         // Just copy them.
         //
-        if (!isSpecial_L1[(unsigned char)*pdstr])
+        if (!isSpecial(L1, *pdstr))
         {
             char *p = pdstr + 1;
-            while (!isSpecial_L1[(unsigned char)*p++])
+            while (!isSpecial(L1, *p++))
             {
                 ; // Nothing.
             }
@@ -1210,7 +1225,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
             // configured.
             //
             char *pEnd = *bufc - 1;
-            if (mudconf.space_compress)
+            if (mudconf.space_compress && (eval & EV_FMAND))
             {
                 while (  oldp <= pEnd
                       && mux_isspace(*pEnd))
@@ -1276,7 +1291,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                 if (  fp
                    && (fp->flags & FN_NOEVAL))
                 {
-                    feval = eval & ~(EV_EVAL|EV_TOP);
+                    feval = eval & ~(EV_EVAL|EV_TOP|EV_STRIP_CURLY);
                 }
                 else
                 {
@@ -1324,6 +1339,10 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                     {
                         safe_noperm(buff, &oldp);
                     }
+                    else if (MuxAlarm.bAlarmed)
+                    {
+                        safe_str("#-1 CPU LIMITED", buff, &oldp);
+                    }
                     else if (ufp)
                     {
                         tstr = atr_get(ufp->obj, ufp->atr, &aowner, &aflags);
@@ -1366,7 +1385,8 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                         // Otherwise, return an error message.
                         //
                         if (  fp->minArgs <= nfargs
-                           && nfargs <= fp->maxArgs)
+                           && nfargs <= fp->maxArgs
+                           && !MuxAlarm.bAlarmed)
                         {
                             fp->fun(buff, &oldp, executor, caller, enactor,
                                     fargs, nfargs, cargs, ncargs);
@@ -1385,6 +1405,10 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                                     "#-1 FUNCTION (%s) EXPECTS %d OR %d ARGUMENTS",
                                     fp->name, fp->minArgs, fp->maxArgs);
                             }
+                            else if (MuxAlarm.bAlarmed)
+                            {
+                                sprintf(mux_scratch, "#-1 CPU LIMITED");
+                            }
                             else
                             {
                                 sprintf(mux_scratch,
@@ -1409,7 +1433,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                 fargs = NULL;
             }
             eval &= ~EV_FCHECK;
-            isSpecial_L1['('] = false;
+            isSpecial(L1, '(') = false;
         }
         else if (*pdstr == '%')
         {
@@ -1435,7 +1459,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
             {
                 pdstr++;
                 ch = *pdstr;
-                unsigned char cType_L2 = isSpecial_L2[(unsigned char)ch];
+                unsigned char cType_L2 = isSpecial(L2, ch);
                 TempPtr = *bufc;
                 int iCode = cType_L2 & 0x7F;
                 if (iCode == 1)
@@ -1621,7 +1645,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                                 nBufferAvailable -= nAttrGotten;
                             }
                         }
-                        else if (ch == '\0')
+                        else if (*pdstr == '\0')
                         {
                             pdstr--;
                         }
@@ -2062,9 +2086,9 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
 
     // Restore Parser Mode.
     //
-    isSpecial_L1[' '] = bSpaceIsSpecialSave;
-    isSpecial_L1['('] = bParenthesisIsSpecialSave;
-    isSpecial_L1['['] = bBracketIsSpecialSave;
+    isSpecial(L1, ' ') = bSpaceIsSpecialSave;
+    isSpecial(L1, '(') = bParenthesisIsSpecialSave;
+    isSpecial(L1, '[') = bBracketIsSpecialSave;
 }
 
 /* ---------------------------------------------------------------------------
@@ -2106,6 +2130,8 @@ void save_and_clear_global_regs
     int preserve_len[]
 )
 {
+    UNUSED_PARAMETER(funcname);
+
     int i;
 
     for (i = 0; i < MAX_GLOBAL_REGS; i++)
@@ -2125,6 +2151,8 @@ void restore_global_regs
     int preserve_len[]
 )
 {
+    UNUSED_PARAMETER(funcname);
+
     int i;
 
     for (i = 0; i < MAX_GLOBAL_REGS; i++)
