@@ -1,6 +1,6 @@
 // mudconf.h
 //
-// $Id: mudconf.h,v 1.15 2003/12/06 01:57:32 sdennis Exp $
+// $Id: mudconf.h,v 1.34 2005/12/29 17:47:40 sdennis Exp $
 //
 
 #ifndef __CONF_H
@@ -8,7 +8,6 @@
 
 #include "alloc.h"
 #include "htab.h"
-#include "mail.h"
 #include "stringutil.h"
 
 #ifndef WIN32
@@ -67,6 +66,7 @@ struct confdata
     bool    quotas;             /* true = have building quotas */
     bool    read_rem_desc;      /* Can the DESCs of nonlocal objs be read? */
     bool    read_rem_name;      /* Can the NAMEs of nonlocal objs be read? */
+    bool    reset_players;      // Reset the maximum player stat.
     bool    robot_speak;        /* true = allow robots to speak */
     bool    run_startup;        // If no, startup attributes aren't processed on load.
     bool    safe_unowned;       /* Are objects not owned by you safe? */
@@ -122,9 +122,26 @@ struct confdata
     int     log_options;        /* What gets logged */
     int     machinecost;        /* One in mc+1 cmds costs 1 penny (POW2-1) */
     int     mail_expiration;    /* Number of days to wait to delete mail */
+    int     mail_per_hour;      // Maximum sent @mail per hour per object.
     int     max_players;        /* Max # of connected players */
     int     min_guests;         // The # we should start nuking at.
     int     nStackLimit;        // Current stack limit.
+#ifdef REALITY_LVLS
+    int     no_levels;          /* Number of reality levels */
+    struct  rlevel_def {
+        char name[9];           /* Rlevel name */
+        RLEVEL value;           /* Rlevel bitmask */
+        char attr[33];          /* desc attribute */
+    } reality_level[32];        /* Reality levels */
+    RLEVEL  def_room_rx;        /* Default room RX level */
+    RLEVEL  def_room_tx;        /* Default room TX level */
+    RLEVEL  def_player_rx;      /* Default player RX level */
+    RLEVEL  def_player_tx;      /* Default player RX level */
+    RLEVEL  def_exit_rx;        /* Default exit RX level */
+    RLEVEL  def_exit_tx;        /* Default exit TX level */
+    RLEVEL  def_thing_rx;       /* Default thing RX level */
+    RLEVEL  def_thing_tx;       /* Default thing TX level */
+#endif /* REALITY_LVLS */
     int     ntfy_nest_lim;      /* Max nesting of notifys */
     int     number_guests;      // number of guest characters allowed.
     int     opencost;           /* cost of @open command */
@@ -136,6 +153,7 @@ struct confdata
     int     paylimit;           /* getting money gets hard over this much */
     int     paystart;           /* new players start with this much money */
     int     player_quota;       /* quota needed to make a robot player */
+    int     pcreate_per_hour;   // Maximum allowed players created per hour */
     int     queue_chunk;        /* # cmds to run from queue when idle */
     int     queuemax;           /* max commands a player may have in queue */
     int     retry_limit;        /* close conn after this many bad logins */
@@ -148,12 +166,14 @@ struct confdata
     int     stack_limit;        /* How big can stacks get? */
     int     start_quota;        /* Quota for new players */
     int     thing_quota;        /* quota needed to make a thing */
-    int     timeslice;          /* How often do we bump people's cmd quotas? */
     int     trace_limit;        /* Max lines of trace output if top-down */
     int     vattr_flags;        /* Attr flags for all user-defined attrs */
+    int     vattr_per_hour;     // Maximum allowed vattrs per hour per object.
     int     waitcost;           /* cost of @wait (refunded when finishes) */
     int     wild_invk_lim;      // Max Regular Expression function calls.
     int     zone_nest_lim;      /* Max nesting of zones */
+    int     restrict_home;	// Special condition to restrict 'home' command
+
     unsigned int    max_cache_size; /* Max size of attribute cache */
     unsigned int    site_chars; // where to truncate site name.
 
@@ -161,10 +181,12 @@ struct confdata
 
     char    guest_prefix[32];   /* Prefix for the guest char's name */
     char    guests_channel[32]; /* Name of guests channel */
+    char    guests_channel_alias[32]; /* Name of guests channel alias */
     char    many_coins[32];     /* name of many coins (ie. "pennies") */
     char    mud_name[32];       /* Name of the mud */
     char    one_coin[32];       /* name of one coin (ie. "penny") */
     char    public_channel[32]; /* Name of public channel */
+    char    public_channel_alias[32]; /* Name of public channel alias */
     char    dump_msg[128];      /* Message displayed when @dump-ing */
     char    fixed_home_msg[128];    /* Message displayed when going home and FIXED */
     char    fixed_tel_msg[128]; /* Message displayed when teleporting and FIXED */
@@ -196,9 +218,12 @@ struct confdata
     char    *status_file;       /* Where to write arg to @shutdown */
     char    *uncompress;        /* program to run to uncompress */
     char    *wizmotd_file;      /* display this file on login to wizards */
+    char    *pid_file;          // file for communicating process id back to ./Startmux
     unsigned char    markdata[8];  /* Masks for marking/unmarking */
-    CLinearTimeDelta max_cmdsecs;  /* Threshhold for real time taken by command */
+    CLinearTimeDelta rpt_cmdsecs;  /* Reporting Threshhold for time taken by command */
+    CLinearTimeDelta max_cmdsecs;  /* Upper Limit for real time taken by command */
     CLinearTimeDelta cache_tick_period; // Minor cycle for cache maintenance.
+    CLinearTimeDelta timeslice;         // How often do we bump people's cmd quotas?
 
     FLAGSET exit_flags;         /* Flags exits start with */
     FLAGSET player_flags;       /* Flags players start with */
@@ -277,13 +302,16 @@ struct statedata
     bool bReadingConfiguration; // are we reading the config file at startup?
     bool bStackLimitReached;    // Was stack slammed?
     bool bStandAlone;           // Are we running in dbconvert mode.
-    bool inpipe;                /* Boolean flag for command piping */
     bool panicking;             // are we in the middle of dying horribly?
     bool shutdown_flag;         /* Should interface be shut down? */
+    bool inpipe;                // Are we collecting output for a pipe?
 #ifndef WIN32
     bool          restarting;   // Are we restarting?
     volatile bool dumping;      // Are we dumping?
-    volatile pid_t dumper;      // PID of dumping process.
+    volatile pid_t dumper;      // PID of dumping process (as returned by fork()).
+    volatile pid_t dumped;      // PID of dumping process (as given by SIGCHLD).
+    bool    write_protect;      // Write-protect against modifications to the
+                                // database during dumps.
 #endif // !WIN32
 
     dbref   curr_enactor;       /* Who initiated the current command */
@@ -311,6 +339,8 @@ struct statedata
     int     nObjEvalNest;       // The nesting level of objeval() invocations.
     int     nStackNest;         // Current stack depth.
     int     nHearNest;          // Current aahear depth.
+    int     pipe_nest_lev;      // Number of piped commands.
+    int     pcreates_this_hour; // Player creations possible this hour.
     int     ntfy_nest_lev;      /* Current nesting of notifys */
     int     record_players;     /* The maximum # of player logged on */
     int     wild_invk_ctr;      // Regular Expression function calls.
@@ -341,7 +371,6 @@ struct statedata
     BADNAME *badname_head;      /* List of disallowed names */
     HELP_DESC *aHelpDesc;       // Table of help files hashes.
     MARKBUF *markbits;          /* temp storage for marking/unmarking */
-    MENT    *mail_list;         /* The mail database */
     OLSTK   *olist;             /* Stack of object lists for nested searches */
     SITE    *suspect_list;      /* Sites that are suspect */
     SITE    *access_list;       /* Access states for sites */
@@ -351,8 +380,8 @@ struct statedata
     CLinearTimeAbsolute dump_counter;   /* Countdown to next db dump */
     CLinearTimeAbsolute events_counter; /* Countdown to next events check */
     CLinearTimeAbsolute idle_counter;   /* Countdown to next idle check */
-    CLinearTimeAbsolute now;            /* What time is it now? */
     CLinearTimeAbsolute start_time;     /* When was MUX started */
+    CLinearTimeAbsolute tThrottleExpired; // How much time is left in this hour of throttling.
 
     CHashTable acache_htab;     // Attribute Cache
     CHashTable attr_name_htab;  /* Attribute names hashtable */
@@ -372,6 +401,14 @@ struct statedata
 #endif // PARSE_TREES
     CHashTable ufunc_htab;      /* Local functions hashtable */
     CHashTable vattr_name_htab; /* User attribute names hashtable */
+
+    CBitField bfNoListens;      // Cache knowledge that there are no ^-Commands.
+    CBitField bfNoCommands;     // Cache knowledge that there are no $-Commands.
+    CBitField bfCommands;       // Cache knowledge that there are $-Commands.
+    CBitField bfListens;        // Cache knowledge that there are ^-Commands.
+
+    CBitField bfReport;         // Used for LROOMS.
+    CBitField bfTraverse;       // Used for LROOMS.
 };
 
 extern STATEDATA mudstate;
