@@ -1,10 +1,10 @@
 // stringutil.cpp -- string utilities.
 //
-// $Id: stringutil.cpp,v 1.58 2005/04/11 17:52:01 sdennis Exp $
+// $Id: stringutil.cpp,v 1.73 2006/01/07 08:00:51 sdennis Exp $
 //
-// MUX 2.3
-// Copyright (C) 1998 through 2003 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.  
+// MUX 2.4
+// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
+// rights not explicitly given are reserved.
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -34,7 +34,7 @@ const bool mux_isprint[256] =
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  // C
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  // D
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  // E
-    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0   // F
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1   // F
 };
 
 const bool mux_isdigit[256] =
@@ -60,6 +60,29 @@ const bool mux_isdigit[256] =
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0   // F
 };
 
+const bool mux_ishex[256] =
+{
+//  0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F
+//
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 0
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 1
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 2
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0,  // 3
+    0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 4
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 5
+    0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 6
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 7
+
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 8
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 9
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // A
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // B
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // C
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // D
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // E
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0   // F
+};
+
 const bool mux_isalpha[256] =
 {
 //  0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F
@@ -307,7 +330,7 @@ const bool mux_issecure[256] =
 };
 
 // Characters which should be escaped for the escape()
-// function: '%\[]{};'.
+// function: '%\[]{};,()^$'.
 //
 const bool mux_isescape[256] =
 {
@@ -315,10 +338,10 @@ const bool mux_isescape[256] =
 //
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 0
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 1
-    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 2
+    0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0,  // 2
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,  // 3
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 4
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0,  // 5
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0,  // 5
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  // 6
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0,  // 7
 
@@ -355,6 +378,28 @@ const bool ANSI_TokenTerminatorTable[256] =
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0   // F
 };
 
+const unsigned char mux_hex2dec[256] =
+{
+//  0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
+//
+    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  // 0
+    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  // 1
+    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  // 2
+    0,  1,  2,  3,  4,  5,  6,  7,  8,  9,  0,  0,  0,  0,  0,  0,  // 3
+    0, 10, 11, 12, 13, 14, 15,  0,  0,  0,  0,  0,  0,  0,  0,  0,  // 4
+    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  // 5
+    0, 10, 11, 12, 13, 14, 15,  0,  0,  0,  0,  0,  0,  0,  0,  0,  // 6
+    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  // 7
+
+    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  // 8
+    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  // A
+    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  // B
+    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  // C
+    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  // D
+    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  // E
+    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0   // F
+};
+
 const unsigned char mux_toupper[256] =
 {
 //   0     1     2     3     4     5     6     7     8     9     A     B     C     D     E     F
@@ -514,7 +559,7 @@ char *strip_ansi(const char *szString, size_t *pnString)
         *pBuffer = '\0';
         return Buffer;
     }
-    int   nString = strlen(szString);
+    size_t nString = strlen(szString);
 
     while (nString)
     {
@@ -580,14 +625,14 @@ char *strip_accents(const char *szString, size_t *pnString)
 #define ANSI_COLOR_INDEX_WHITE     7
 #define ANSI_COLOR_INDEX_DEFAULT   8
 
-const ANSI_ColorState acsRestingStates[3] =
+static const ANSI_ColorState acsRestingStates[3] =
 {
     {true,  false, false, false, false, ANSI_COLOR_INDEX_DEFAULT, ANSI_COLOR_INDEX_DEFAULT},
     {false, false, false, false, false, ANSI_COLOR_INDEX_WHITE,   ANSI_COLOR_INDEX_DEFAULT},
     {true,  false, false, false, false, ANSI_COLOR_INDEX_DEFAULT, ANSI_COLOR_INDEX_DEFAULT}
 };
 
-void ANSI_Parse_m(ANSI_ColorState *pacsCurrent, int nANSI, const char *pANSI,
+static void ANSI_Parse_m(ANSI_ColorState *pacsCurrent, int nANSI, const char *pANSI,
                   bool *pbSawNormal)
 {
     // If the last character isn't an 'm', then it's an ANSI sequence we
@@ -613,7 +658,7 @@ void ANSI_Parse_m(ANSI_ColorState *pacsCurrent, int nANSI, const char *pANSI,
         {
             p++;
         }
-        int nLen = p - pANSI + 1;
+        size_t nLen = p - pANSI + 1;
         if (p[0] == 'm' || p[0] == ';')
         {
             // We have an attribute.
@@ -705,7 +750,7 @@ void ANSI_Parse_m(ANSI_ColorState *pacsCurrent, int nANSI, const char *pANSI,
 // Generate the minimal ANSI sequence that will transition from one color state
 // to another.
 //
-char *ANSI_TransitionColorBinary
+static char *ANSI_TransitionColorBinary
 (
     ANSI_ColorState *acsCurrent,
     const ANSI_ColorState *pcsNext,
@@ -793,7 +838,7 @@ char *ANSI_TransitionColorBinary
 // Generate the minimal MU ANSI %-sequence that will transition from one color state
 // to another.
 //
-char *ANSI_TransitionColorEscape
+static char *ANSI_TransitionColorEscape
 (
     ANSI_ColorState *acsCurrent,
     ANSI_ColorState *acsNext,
@@ -801,8 +846,8 @@ char *ANSI_TransitionColorEscape
 )
 {
     static char Buffer[ANSI_MAXIMUM_ESCAPE_TRANSITION_LENGTH+1];
-    static char cForegroundColors[9] = "xrgybmcw";
-    static char cBackgroundColors[9] = "XRGYBMCW";
+    static const char cForegroundColors[9] = "xrgybmcw";
+    static const char cBackgroundColors[9] = "XRGYBMCW";
 
     if (memcmp(acsCurrent, acsNext, sizeof(ANSI_ColorState)) == 0)
     {
@@ -811,7 +856,7 @@ char *ANSI_TransitionColorEscape
         return Buffer;
     }
     ANSI_ColorState tmp = *acsCurrent;
-    char *p = Buffer;
+    int  i = 0;
 
     // Do we need to go through the normal state?
     //
@@ -823,37 +868,56 @@ char *ANSI_TransitionColorEscape
        || (  tmp.iForeground != ANSI_COLOR_INDEX_DEFAULT
           && acsNext->iForeground == ANSI_COLOR_INDEX_DEFAULT))
     {
-        memcpy(p, "%xn", 3); p += 3;
+        Buffer[i  ] = '%';
+        Buffer[i+1] = 'x';
+        Buffer[i+2] = 'n';
+        i = i + 3;
         tmp = acsRestingStates[ANSI_ENDGOAL_NORMAL];
     }
     if (tmp.bHighlite != acsNext->bHighlite)
     {
-        memcpy(p, "%xh", 3); p += 3;
+        Buffer[i  ] = '%';
+        Buffer[i+1] = 'x';
+        Buffer[i+2] = 'h';
+        i = i + 3;
     }
     if (tmp.bUnder != acsNext->bUnder)
     {
-        memcpy(p, "%xu", 3); p += 3;
+        Buffer[i  ] = '%';
+        Buffer[i+1] = 'x';
+        Buffer[i+2] = 'u';
+        i = i + 3;
     }
     if (tmp.bBlink != acsNext->bBlink)
     {
-        memcpy(p, "%xf", 3); p += 3;
+        Buffer[i  ] = '%';
+        Buffer[i+1] = 'x';
+        Buffer[i+2] = 'f';
+        i = i + 3;
     }
     if (tmp.bInverse != acsNext->bInverse)
     {
-        memcpy(p, "%xi", 3); p += 3;
+        Buffer[i  ] = '%';
+        Buffer[i+1] = 'x';
+        Buffer[i+2] = 'i';
+        i = i + 3;
     }
     if (tmp.iForeground != acsNext->iForeground)
     {
-        memcpy(p, "%x", 2); p += 2;
-        *p++ = cForegroundColors[acsNext->iForeground];
+        Buffer[i  ] = '%';
+        Buffer[i+1] = 'x';
+        Buffer[i+2] = cForegroundColors[acsNext->iForeground];
+        i = i + 3;
     }
     if (tmp.iBackground != acsNext->iBackground)
     {
-        memcpy(p, "%x", 2); p += 2;
-        *p++ = cBackgroundColors[acsNext->iBackground];
+        Buffer[i  ] = '%';
+        Buffer[i+1] = 'x';
+        Buffer[i+2] = cBackgroundColors[acsNext->iBackground];
+        i = i + 3;
     }
-    *p = '\0';
-    *nTransition = p - Buffer;
+    Buffer[i] = '\0';
+    *nTransition = i;
     return Buffer;
 }
 
@@ -1240,7 +1304,7 @@ typedef struct
     char *p;
 } LITERAL_STRING_STRUCT;
 
-LITERAL_STRING_STRUCT MU_Substitutes[] =
+static LITERAL_STRING_STRUCT MU_Substitutes[] =
 {
     { 1, " "  },  // 0
     { 1, " "  },  // 1
@@ -1318,7 +1382,7 @@ char *translate_string(const char *szString, bool bConvert)
         *pTranslatedString = '\0';
         return szTranslatedString;
     }
-    int   nString = strlen(szString);
+    size_t nString = strlen(szString);
 
     ANSI_ColorState acsCurrent;
     ANSI_ColorState acsPrevious;
@@ -1639,7 +1703,7 @@ char *replace_string(const char *old, const char *new0, const char *s)
     {
         return NULL;
     }
-    int olen = strlen(old);
+    size_t olen = strlen(old);
     char *result = alloc_lbuf("replace_string");
     char *r = result;
     while (*s)
@@ -1647,11 +1711,12 @@ char *replace_string(const char *old, const char *new0, const char *s)
         // Find next occurrence of the first character of OLD string.
         //
         const char *p;
-        if (olen && (p = strchr(s, old[0])))
+        if (  olen
+           && (p = strchr(s, old[0])))
         {
             // Copy up to the next occurrence of the first char of OLD.
             //
-            int n = p - s;
+            size_t n = p - s;
             if (n)
             {
                 safe_copy_buf(s, n, result, &r);
@@ -1713,7 +1778,7 @@ char *replace_tokens
         {
             // Copy up to the next occurrence of the first character.
             //
-            int n = p - s;
+            size_t n = p - s;
             if (n)
             {
                 safe_copy_buf(s, n, result, &r);
@@ -1812,10 +1877,7 @@ bool minmatch(char *str, char *target, int min)
 char *StringCloneLen(const char *str, size_t nStr)
 {
     char *buff = (char *)MEMALLOC(nStr+1);
-    if (ISOUTOFMEMORY(buff))
-    {
-        return 0;
-    }
+    ISOUTOFMEMORY(buff);
     memcpy(buff, str, nStr);
     buff[nStr] = '\0';
     return buff;
@@ -1836,10 +1898,7 @@ char *StringClone(const char *str)
 char *BufferCloneLen(const char *pBuffer, unsigned int nBuffer)
 {
     char *buff = (char *)MEMALLOC(nBuffer);
-    if (ISOUTOFMEMORY(buff))
-    {
-        return 0;
-    }
+    ISOUTOFMEMORY(buff);
     memcpy(buff, pBuffer, nBuffer);
     return buff;
 }
@@ -1864,7 +1923,10 @@ void safe_copy_str(const char *src, char *buff, char **bufp, int nSizeOfBuffer)
 
 void safe_copy_str_lbuf(const char *src, char *buff, char **bufp)
 {
-    if (src == NULL) return;
+    if (src == NULL)
+    {
+        return;
+    }
 
     char *tp = *bufp;
     char *maxtp = buff + LBUF_SIZE - 1;
@@ -1875,9 +1937,9 @@ void safe_copy_str_lbuf(const char *src, char *buff, char **bufp)
     *bufp = tp;
 }
 
-int safe_copy_buf(const char *src, int nLen, char *buff, char **bufc)
+size_t safe_copy_buf(const char *src, size_t nLen, char *buff, char **bufc)
 {
-    int left = LBUF_SIZE - (*bufc - buff) - 1;
+    size_t left = LBUF_SIZE - (*bufc - buff) - 1;
     if (left < nLen)
     {
         nLen = left;
@@ -1887,11 +1949,11 @@ int safe_copy_buf(const char *src, int nLen, char *buff, char **bufc)
     return nLen;
 }
 
-int safe_fill(char *buff, char **bufc, char chFill, int nSpaces)
+size_t safe_fill(char *buff, char **bufc, char chFill, size_t nSpaces)
 {
     // Check for buffer limits.
     //
-    int nBufferAvailable = LBUF_SIZE - (*bufc - buff) - 1;
+    size_t nBufferAvailable = LBUF_SIZE - (*bufc - buff) - 1;
     if (nSpaces > nBufferAvailable)
     {
         nSpaces = nBufferAvailable;
@@ -1957,7 +2019,7 @@ const char Digits100[201] =
 061626364656667686960717273747576777879708182838485868788898\
 09192939495969798999";
 
-int mux_ltoa(long val, char *buf)
+size_t mux_ltoa(long val, char *buf)
 {
     char *p = buf;
 
@@ -1966,7 +2028,7 @@ int mux_ltoa(long val, char *buf)
         *p++ = '-';
         val = -val;
     }
-    unsigned int uval = (unsigned int)val;
+    unsigned long uval = (unsigned long)val;
 
     char *q = p;
 
@@ -1985,7 +2047,7 @@ int mux_ltoa(long val, char *buf)
         *p++ = *(z+1);
     }
 
-    int nLength = p - buf;
+    size_t nLength = p - buf;
     *p-- = '\0';
 
     // The digits are in reverse order with a possible leading '-'
@@ -2021,11 +2083,11 @@ char *mux_ltoa_t(long val)
 void safe_ltoa(long val, char *buff, char **bufc)
 {
     static char temp[12];
-    int n = mux_ltoa(val, temp);
+    size_t n = mux_ltoa(val, temp);
     safe_copy_buf(temp, n, buff, bufc);
 }
 
-int mux_i64toa(INT64 val, char *buf)
+size_t mux_i64toa(INT64 val, char *buf)
 {
     char *p = buf;
 
@@ -2053,7 +2115,7 @@ int mux_i64toa(INT64 val, char *buf)
         *p++ = *(z+1);
     }
 
-    int nLength = p - buf;
+    size_t nLength = p - buf;
     *p-- = '\0';
 
     // The digits are in reverse order with a possible leading '-'
@@ -2091,7 +2153,7 @@ char *mux_i64toa_t(INT64 val)
 void safe_i64toa(INT64 val, char *buff, char **bufc)
 {
     static char temp[22];
-    int n = mux_i64toa(val, temp);
+    size_t n = mux_i64toa(val, temp);
     safe_copy_buf(temp, n, buff, bufc);
 }
 
@@ -2228,7 +2290,7 @@ bool ParseFloat(PARSE_FLOAT_RESULT *pfr, const char *str, bool bStrict)
 
     // Parse Input
     //
-    unsigned char ch0;
+    unsigned char ch;
     pfr->pMeat = str;
     if (  !mux_isdigit(*str)
        && *str != '.')
@@ -2255,16 +2317,16 @@ bool ParseFloat(PARSE_FLOAT_RESULT *pfr, const char *str, bool bStrict)
         {
             // Look for three magic strings.
             //
-            unsigned char ch0 = mux_toupper(str[0]);
-            if (ch0 == 'I')
+            ch = mux_toupper(str[0]);
+            if (ch == 'I')
             {
                 // Could be 'Inf' or 'Ind'
                 //
-                ch0 = mux_toupper(str[1]);
-                if (ch0 == 'N')
+                ch = mux_toupper(str[1]);
+                if (ch == 'N')
                 {
-                    ch0 = mux_toupper(str[2]);
-                    if (ch0 == 'F')
+                    ch = mux_toupper(str[2]);
+                    if (ch == 'F')
                     {
                         // Inf
                         //
@@ -2279,7 +2341,7 @@ bool ParseFloat(PARSE_FLOAT_RESULT *pfr, const char *str, bool bStrict)
                         str += 3;
                         goto LastSpaces;
                     }
-                    else if (ch0 == 'D')
+                    else if (ch == 'D')
                     {
                         // Ind
                         //
@@ -2289,15 +2351,15 @@ bool ParseFloat(PARSE_FLOAT_RESULT *pfr, const char *str, bool bStrict)
                     }
                 }
             }
-            else if (ch0 == 'N')
+            else if (ch == 'N')
             {
                 // Could be 'Nan'
                 //
-                ch0 = mux_toupper(str[1]);
-                if (ch0 == 'A')
+                ch = mux_toupper(str[1]);
+                if (ch == 'A')
                 {
-                    ch0 = mux_toupper(str[2]);
-                    if (ch0 == 'N')
+                    ch = mux_toupper(str[2]);
+                    if (ch == 'N')
                     {
                         // Nan
                         //
@@ -2340,8 +2402,8 @@ bool ParseFloat(PARSE_FLOAT_RESULT *pfr, const char *str, bool bStrict)
         return false;
     }
 
-    ch0 = mux_toupper(*str);
-    if (ch0 == 'E')
+    ch = mux_toupper(*str);
+    if (ch == 'E')
     {
         // There is an exponent portion.
         //
@@ -2466,7 +2528,7 @@ double mux_atof(char *szString, bool bStrict)
     }
 
     const char *p = pfr.pMeat;
-    int n = pfr.nMeat;
+    size_t n = pfr.nMeat;
 
     // We need to protect certain libraries from going nuts from being
     // force fed lots of ASCII.
@@ -2490,9 +2552,6 @@ double mux_atof(char *szString, bool bStrict)
     return ret;
 }
 
-extern char *mux_dtoa(double d, int mode, int nRequest, int *iDecimalPoint,
-                       int *sign, char **rve);
-
 char *mux_ftoa(double r, bool bRounded, int frac)
 {
     static char buffer[100];
@@ -3144,7 +3203,7 @@ int DCL_CDECL mux_vsnprintf(char *buff, int count, const char *fmt, va_list va)
 //
 int GetLineTrunc(char *Buffer, size_t nBuffer, FILE *fp)
 {
-    int lenBuffer = 0;
+    size_t lenBuffer = 0;
     if (fgets(Buffer, nBuffer, fp))
     {
         lenBuffer = strlen(Buffer);
@@ -3160,7 +3219,7 @@ int GetLineTrunc(char *Buffer, size_t nBuffer, FILE *fp)
         // end of the line.
         //
         char TruncBuffer[SBUF_SIZE];
-        int lenTruncBuffer;
+        size_t lenTruncBuffer;
         do
         {
             if (!fgets(TruncBuffer, sizeof(TruncBuffer), fp))
@@ -3184,7 +3243,7 @@ int GetLineTrunc(char *Buffer, size_t nBuffer, FILE *fp)
 // the full Boyer-Moore would make more sense.
 //
 #define BMH_LARGE 32767
-void BMH_Prepare(BMH_State *bmhs, int nPat, char *pPat)
+void BMH_Prepare(BMH_State *bmhs, int nPat, const char *pPat)
 {
     if (nPat <= 0)
     {
@@ -3209,7 +3268,7 @@ void BMH_Prepare(BMH_State *bmhs, int nPat, char *pPat)
     bmhs->m_d[(unsigned char)chLastPat] = BMH_LARGE;
 }
 
-int BMH_Execute(BMH_State *bmhs, int nPat, char *pPat, int nSrc, char *pSrc)
+int BMH_Execute(BMH_State *bmhs, int nPat, const char *pPat, int nSrc, const char *pSrc)
 {
     if (nPat <= 0)
     {
@@ -3227,7 +3286,7 @@ int BMH_Execute(BMH_State *bmhs, int nPat, char *pPat, int nSrc, char *pSrc)
         }
         i -= BMH_LARGE;
         int j = nPat - 1;
-        char *s = pSrc + (i - j);
+        const char *s = pSrc + (i - j);
         while (--j >= 0 && s[j] == pPat[j])
         {
             ; // Nothing.
@@ -3240,14 +3299,14 @@ int BMH_Execute(BMH_State *bmhs, int nPat, char *pPat, int nSrc, char *pSrc)
     return -1;
 }
 
-int BMH_StringSearch(int nPat, char *pPat, int nSrc, char *pSrc)
+int BMH_StringSearch(int nPat, const char *pPat, int nSrc, const char *pSrc)
 {
     BMH_State bmhs;
     BMH_Prepare(&bmhs, nPat, pPat);
     return BMH_Execute(&bmhs, nPat, pPat, nSrc, pSrc);
 }
 
-void BMH_PrepareI(BMH_State *bmhs, int nPat, char *pPat)
+void BMH_PrepareI(BMH_State *bmhs, int nPat, const char *pPat)
 {
     if (nPat <= 0)
     {
@@ -3274,7 +3333,7 @@ void BMH_PrepareI(BMH_State *bmhs, int nPat, char *pPat)
     bmhs->m_d[mux_tolower(chLastPat)] = BMH_LARGE;
 }
 
-int BMH_ExecuteI(BMH_State *bmhs, int nPat, char *pPat, int nSrc, char *pSrc)
+int BMH_ExecuteI(BMH_State *bmhs, int nPat, const char *pPat, int nSrc, const char *pSrc)
 {
     if (nPat <= 0)
     {
@@ -3292,7 +3351,7 @@ int BMH_ExecuteI(BMH_State *bmhs, int nPat, char *pPat, int nSrc, char *pSrc)
         }
         i -= BMH_LARGE;
         int j = nPat - 1;
-        char *s = pSrc + (i - j);
+        const char *s = pSrc + (i - j);
         while (  --j >= 0
               && mux_toupper(s[j]) == mux_toupper(pPat[j]))
         {
@@ -3306,7 +3365,7 @@ int BMH_ExecuteI(BMH_State *bmhs, int nPat, char *pPat, int nSrc, char *pSrc)
     return -1;
 }
 
-int BMH_StringSearchI(int nPat, char *pPat, int nSrc, char *pSrc)
+int BMH_StringSearchI(int nPat, const char *pPat, int nSrc, const char *pSrc)
 {
     BMH_State bmhs;
     BMH_PrepareI(&bmhs, nPat, pPat);
@@ -3319,10 +3378,11 @@ int BMH_StringSearchI(int nPat, char *pPat, int nSrc, char *pSrc)
 // Add an article rule to the ruleset.
 //
 
-extern void DCL_CDECL cf_log_syntax(dbref player, char *cmd, const char *fmt, ...);
-
 CF_HAND(cf_art_rule)
 {
+    UNUSED_PARAMETER(pExtra);
+    UNUSED_PARAMETER(nExtra);
+
     char* pCurrent = str;
 
     while (mux_isspace(*pCurrent))
