@@ -1,10 +1,10 @@
 // svdhash.cpp -- CHashPage, CHashFile, CHashTable modules.
 //
-// $Id: svdhash.cpp,v 1.20 2005/08/06 17:39:28 sdennis Exp $
+// $Id: svdhash.cpp,v 1.43 2006/01/07 07:49:53 sdennis Exp $
 //
-// MUX 2.3
-// Copyright (C) 1998 through 2003 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.  
+// MUX 2.4
+// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
+// rights not explicitly given are reserved.
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -292,7 +292,7 @@ const int Primes[NUMBER_OF_PRIMES] =
     977, 983, 991, 997, 1009, 1013, 1019, 1021, 1031, 1033, 1039, 0
 };
 
-void ChoosePrimes(int TableSize, HP_HEAPOFFSET HashPrimes[16])
+static void ChoosePrimes(int TableSize, HP_HEAPOFFSET HashPrimes[16])
 {
     int LargestPrime = TableSize/2;
     if (LargestPrime > Primes[NUMBER_OF_PRIMES-2])
@@ -389,7 +389,7 @@ CHashPage::~CHashPage(void)
 {
     if (m_pPage)
     {
-        delete m_pPage;
+        delete [] m_pPage;
         m_pPage = 0;
     }
 }
@@ -439,9 +439,11 @@ void CHashPage::GetStats
     // If we have records to talk about, or even if we are trying to reserve
     // space, then do the math.
     //
-    if (nExtra != 0 || nCount != 0)
+    if (  nExtra != 0
+       || nCount != 0)
     {
-        UINT32 nSpace = ((unsigned char *)m_pTrailer) - ((unsigned char *)m_pDirectory);
+        UINT32 nSpace = (UINT32)( ((unsigned char *)m_pTrailer)
+                                - ((unsigned char *)m_pDirectory));
         UINT32 nMinDirSize = nCount;
         UINT32 nMaxDirSize = (nSpace - nSize)/sizeof(HP_HEAPOFFSET);
 
@@ -505,7 +507,7 @@ void CHashPage::Empty(HP_DIRINDEX arg_nDepth, UINT32 arg_nHashGroup, HP_DIRINDEX
     // Setup initial free list.
     //
     HP_PHEAPNODE pNode = (HP_PHEAPNODE)m_pHeapStart;
-    pNode->nBlockSize = m_pHeapEnd - m_pHeapStart;
+    pNode->nBlockSize = (HP_HEAPLENGTH)(m_pHeapEnd - m_pHeapStart);
     pNode->u.oNext = HP_NIL_OFFSET;
     m_pHeader->m_oFreeList = 0; // This is intentionally zero (i.e., m_pHeapStart - m_pHeapStart).
 }
@@ -700,8 +702,9 @@ int CHashPage::Insert(HP_HEAPLENGTH nRecord, UINT32 nHash, void *pRecord)
 //        directory index or HP_DIR_EMPTY if no hash keys are found.
 //
 //        Call iDir = FindFirstKey(hash) the first time, and then call
-//        iDir = FindNextKey(iDir, hash) every time after than until iDir == HP_DIR_EMPTY
-//        to interate through all the records with the desired hash key.
+//        iDir = FindNextKey(iDir, hash) every time after than until
+//        iDir == HP_DIR_EMPTY to interate through all the records with the
+//        desired hash key.
 //
 HP_DIRINDEX CHashPage::FindFirstKey(UINT32 nHash, unsigned int *numchecks)
 {
@@ -713,14 +716,42 @@ HP_DIRINDEX CHashPage::FindFirstKey(UINT32 nHash, unsigned int *numchecks)
         return HP_DIR_EMPTY;
 #endif // HP_PROTECTION
 
-    int nDirSize = m_pHeader->m_nDirSize;
+    const int nDirSize = m_pHeader->m_nDirSize;
 
     // Where do we begin our first probe?
     //
-    int di = m_pHeader->m_Primes[nHash & 15];
     int iDir = (nHash >> 4) % nDirSize;
-    m_nProbesLeft = nDirSize;
     int sOffset = m_pDirectory[iDir];
+    if ((unsigned int)sOffset < HP_DIR_DELETED)
+    {
+        HP_PHEAPNODE pNode = (HP_PHEAPNODE)(m_pHeapStart + sOffset);
+        if (pNode->u.s.nHash == nHash)
+        {
+            m_nProbesLeft = nDirSize - 1;
+            *numchecks = 1;
+            return iDir;
+        }
+    }
+    else if (HP_DIR_EMPTY == sOffset)
+    {
+        m_nProbesLeft = nDirSize;
+        *numchecks = 0;
+        return HP_DIR_EMPTY;
+    }
+
+    //    HP_DIR_DELETED == sOffset
+    // || pNode->u.s.nHash != nHash
+
+    m_nProbesLeft = nDirSize - 1;
+    int di = m_pHeader->m_Primes[nHash & 15];
+
+    iDir += di;
+    if (iDir >= nDirSize)
+    {
+        iDir -= nDirSize;
+    }
+    sOffset = m_pDirectory[iDir];
+
     while (sOffset != HP_DIR_EMPTY)
     {
         m_nProbesLeft--;
@@ -983,9 +1014,9 @@ bool CHashPage::Split(CHashPage &hp0, CHashPage &hp1)
         }
     }
 #if 0
-    UINT32 nRecords0, nRecords1;
-    UINT32 nAllocatedSize0, nAllocatedSize1;
-    UINT32 temp;
+    int nRecords0, nRecords1;
+    HP_HEAPLENGTH nAllocatedSize0, nAllocatedSize1;
+    int    temp;
     hp0.GetStats(0, &nRecords0, &nAllocatedSize0, &temp);
     hp1.GetStats(0, &nRecords1, &nAllocatedSize1, &temp);
     Log.tinyprintf("Split (%d %d) page into (%d %d) and (%d %d)" ENDLINE,
@@ -1022,7 +1053,7 @@ void CHashPage::GetRange
 bool CHashPage::WritePage(HANDLE hFile, HF_FILEOFFSET oWhere)
 {
     cs_dbwrites++;
-    for ( ; ; Sleep(250))
+    for ( ; ; MuxAlarm.Sleep(time_250ms))
     {
         if (SetFilePointer(hFile, oWhere, 0, FILE_BEGIN) == 0xFFFFFFFFUL)
         {
@@ -1047,7 +1078,7 @@ bool CHashPage::ReadPage(HANDLE hFile, HF_FILEOFFSET oWhere)
 {
     cs_dbreads++;
     SetFixedPointers();
-    for ( ; ; Sleep(250))
+    for ( ; ; MuxAlarm.Sleep(time_250ms))
     {
         if (SetFilePointer(hFile, oWhere, 0, FILE_BEGIN) == 0xFFFFFFFFUL)
         {
@@ -1074,7 +1105,7 @@ bool CHashPage::WritePage(HANDLE hFile, HF_FILEOFFSET oWhere)
 {
     cs_dbwrites++;
     int cnt = 60;
-    for ( ; cnt; sleep(1), cnt--)
+    for ( ; cnt; MuxAlarm.Sleep(time_1s), cnt--)
     {
 #ifdef HAVE_PWRITE
         int cc = pwrite(hFile, m_pPage, m_nPageSize, oWhere);
@@ -1114,7 +1145,7 @@ bool CHashPage::ReadPage(HANDLE hFile, HF_FILEOFFSET oWhere)
     cs_dbreads++;
     SetFixedPointers();
     int cnt = 60;
-    for ( ; cnt; sleep(1), cnt--)
+    for ( ; cnt; MuxAlarm.Sleep(time_1s), cnt--)
     {
 #ifdef HAVE_PREAD
         int cc = pread(hFile, m_pPage, m_nPageSize, oWhere);
@@ -1251,8 +1282,6 @@ CHashFile::CHashFile(void)
 {
     SeedRandomNumberGenerator();
     m_Cache = NULL;
-    m_hpCacheLookup = NULL;
-    m_nhpCacheLookup = 0;
     m_nCache = 0;
     Init();
 }
@@ -1264,9 +1293,8 @@ void CHashFile::Init(void)
     m_nDir = 0;
     m_nDirDepth = 0;
     m_pDir = NULL;
-    m_iAgeNext = 0;
+    m_hpCacheLookup = NULL;
     iCache = 0;
-    m_iLastEmpty = 0;
     m_iLastFlushed = 0;
 }
 
@@ -1280,7 +1308,10 @@ void CHashFile::WriteDirectory(void)
     WriteFile(m_hDirFile, m_pDir, sizeof(HF_FILEOFFSET)*m_nDir, &nWritten, 0);
     SetEndOfFile(m_hDirFile);
 #ifdef DO_COMMIT
-    FlushFileBuffers(m_hDirFile);
+    if (!mudstate.bStandAlone)
+    {
+        FlushFileBuffers(m_hDirFile);
+    }
 #endif // DO_COMMIT
 }
 #else // WIN32
@@ -1296,28 +1327,47 @@ void CHashFile::WriteDirectory(void)
 #endif // HAVE_PWRITE
     //SetEndOfFile(m_hDirFile);
 #ifdef DO_COMMIT
-    fsync(m_hDirFile);
+    if (!mudstate.bStandAlone)
+    {
+        fsync(m_hDirFile);
+    }
 #endif // DO_COMMIT
 }
 #endif // WIN32
 
-bool CHashFile::EmptyDirectory(void)
+bool CHashFile::InitializeDirectory(unsigned int n)
 {
     if (m_pDir)
     {
         MEMFREE(m_pDir);
         m_pDir = NULL;
     }
+    if (m_hpCacheLookup)
+    {
+        MEMFREE(m_hpCacheLookup);
+        m_hpCacheLookup = NULL;
+    }
 
-    m_nDir = 2;
-    m_nDirDepth = 1;
+    m_nDir = n;
+    m_nDirDepth = 0;
+    n >>= 1;
+    while (n)
+    {
+        m_nDirDepth++;
+        n >>= 1;
+    }
 
     m_pDir = (HF_FILEOFFSET *)MEMALLOC(sizeof(HF_FILEOFFSET)*m_nDir);
-    if (ISOUTOFMEMORY(m_pDir))
+    ISOUTOFMEMORY(m_pDir);
+    m_hpCacheLookup = new int[m_nDir];
+    ISOUTOFMEMORY(m_hpCacheLookup);
+
+    for (unsigned int i = 0; i < m_nDir; i++)
     {
-        return false;
+        m_pDir[i] = 0xFFFFFFFFUL;
+        m_hpCacheLookup[i] = -1;
     }
-    m_pDir[0] = m_pDir[1] = 0xFFFFFFFFUL;
+
     return true;
 }
 
@@ -1352,7 +1402,7 @@ bool CHashFile::CreateFileSet(const char *szDirFile, const char *szPageFile)
 
     // Create empty structures in memory and write them out.
     //
-    if (!EmptyDirectory())
+    if (!InitializeDirectory(2))
     {
         return false;
     }
@@ -1363,7 +1413,11 @@ bool CHashFile::CreateFileSet(const char *szDirFile, const char *szPageFile)
         return false;
     }
     m_Cache[iCache].m_hp.Empty(0, 0UL, 100);
-    m_pDir[0] = m_pDir[1] = m_Cache[iCache].m_o = 0UL;
+    m_Cache[iCache].m_o = 0UL;
+
+    m_pDir[0] = m_pDir[1] = m_Cache[iCache].m_o;
+    m_hpCacheLookup[0] = m_hpCacheLookup[1] = iCache;
+
     m_Cache[iCache].m_iState = HF_CACHE_UNPROTECTED;
 
     oEndOfFile = HF_SIZEOF_PAGE;
@@ -1379,17 +1433,33 @@ bool CHashFile::RebuildDirectory(void)
 {
     // Initialize in-memory page directory
     //
-    if (!EmptyDirectory())
+    if (!InitializeDirectory(2))
     {
         return false;
     }
 
     // Re-build the directory from CHashPages.
     //
-    int Hits = 0;
     for (UINT32 oPage = 0; oPage < oEndOfFile; oPage += HF_SIZEOF_PAGE)
     {
-        int iCache = ReadCache(oPage, &Hits);
+        int iCache;
+        if ((iCache = AllocateEmptyPage(0, NULL)) < 0)
+        {
+            Log.WriteString("CHashFile::RebuildDirectory.  AllocateEmptyPage failed. DB DAMAGE." ENDLINE);
+            return false;
+        }
+
+        if (m_Cache[iCache].m_hp.ReadPage(m_hPageFile, oPage))
+        {
+            m_Cache[iCache].m_o = oPage;
+            m_Cache[iCache].m_iState = HF_CACHE_CLEAN;
+            ResetAge(iCache);
+        }
+        else
+        {
+            Log.WriteString("CHashFile::RebuildDirectory.  ReadPage failed to get the page. DB DAMAGE." ENDLINE);
+        }
+
         UINT32 nPageDepth = m_Cache[iCache].m_hp.GetDepth();
         while (m_nDirDepth < nPageDepth)
         {
@@ -1408,6 +1478,7 @@ bool CHashFile::RebuildDirectory(void)
                 return false;
             }
             m_pDir[nStart] = oPage;
+            m_hpCacheLookup[nStart] = iCache;
         }
     }
 
@@ -1436,20 +1507,9 @@ bool CHashFile::ReadDirectory(void)
     {
         return false;
     }
-    m_nDir = (int)(cc / HF_SIZEOF_FILEOFFSET);
-    m_nDirDepth = 0;
-    m_pDir = (HF_FILEOFFSET *)MEMALLOC(sizeof(HF_FILEOFFSET)*m_nDir);
-    if (ISOUTOFMEMORY(m_pDir))
-    {
-        return false;
-    }
-    int n = m_nDir;
-    n >>= 1;
-    while (n)
-    {
-        m_nDirDepth++;
-        n >>= 1;
-    }
+
+    InitializeDirectory(cc / HF_SIZEOF_FILEOFFSET);
+
 #ifdef WIN32
     cc = SetFilePointer(m_hDirFile, 0, 0, FILE_BEGIN);
     DWORD nRead;
@@ -1476,17 +1536,18 @@ void CHashFile::InitCache(int nCachePages)
     //
     m_nCache = nCachePages;
     m_Cache = new HF_CACHE[m_nCache];
+    ISOUTOFMEMORY(m_Cache);
     for (int i = 0; i < m_nCache; i++)
     {
         m_Cache[i].m_hp.Allocate(HF_SIZEOF_PAGE);
         m_Cache[i].m_iState = HF_CACHE_EMPTY;
-        m_Cache[i].m_o = 0L;
-        m_Cache[i].m_Age = 0;
+        m_Cache[i].m_o = 0UL;
+        m_Cache[i].m_iYounger = i-1;
+        m_Cache[i].m_iOlder   = i+1;
     }
-
-    m_nhpCacheLookup = 2*m_nCache;
-    m_hpCacheLookup = new int[m_nhpCacheLookup];
-    memset(m_hpCacheLookup, 0, sizeof(int)*m_nhpCacheLookup);
+    m_Cache[0].m_iYounger = m_nCache-1;
+    m_Cache[m_nCache-1].m_iOlder = 0;
+    m_iOldest = 0;
 }
 
 int CHashFile::Open(const char *szDirFile, const char *szPageFile, int nCachePages)
@@ -1606,22 +1667,30 @@ void CHashFile::Sync(void)
         bool bAllFlushed = true;
         for (int i = 0; i < m_nCache; i++)
         {
-            bAllFlushed &= FlushCache(i);
+            if (!FlushCache(i))
+            {
+                bAllFlushed = false;
+            }
         }
         if (!bAllFlushed)
         {
             Log.WriteString("CHashFile::Sync. Could not flush all the pages. DB DAMAGE." ENDLINE);
         }
+
 #ifdef DO_COMMIT
+        if (!mudstate.bStandAlone)
+        {
 #ifdef WIN32
-        FlushFileBuffers(m_hPageFile);
+            FlushFileBuffers(m_hPageFile);
 #else // WIN32
-        fsync(m_hPageFile);
+            fsync(m_hPageFile);
 #endif // WIN32
+        }
 #endif // DO_COMMIT
     }
 #ifdef DO_COMMIT
-    if (m_hDirFile != INVALID_HANDLE_VALUE)
+    if (  m_hDirFile != INVALID_HANDLE_VALUE
+       && !mudstate.bStandAlone)
     {
 #ifdef WIN32
         FlushFileBuffers(m_hDirFile);
@@ -1642,6 +1711,12 @@ void CHashFile::CloseAll(void)
             MEMFREE(m_pDir);
             m_pDir = NULL;
         }
+        if (m_hpCacheLookup)
+        {
+            delete [] m_hpCacheLookup;
+            m_hpCacheLookup = NULL;
+        }
+
 #ifdef WIN32
         CloseHandle(m_hPageFile);
 #else // WIN32
@@ -1667,12 +1742,6 @@ void CHashFile::FinalCache(void)
         m_Cache = NULL;
         m_nCache = 0;
     }
-    if (m_hpCacheLookup)
-    {
-        delete [] m_hpCacheLookup;
-        m_hpCacheLookup = NULL;
-        m_nhpCacheLookup = 0;
-    }
 }
 
 CHashFile::~CHashFile(void)
@@ -1692,8 +1761,7 @@ bool CHashFile::Insert(HP_HEAPLENGTH nRecord, UINT32 nHash, void *pRecord)
             Log.WriteString("CHashFile::Insert - iFileDir out of range." ENDLINE);
             return false;
         }
-        HF_FILEOFFSET oPage = m_pDir[iFileDir];
-        iCache = ReadCache(oPage, &cs_whits);
+        iCache = ReadCache(iFileDir, &cs_whits);
         if (iCache < 0)
         {
             Log.WriteString("CHashFile::Insert - Page wasn't valid." ENDLINE);
@@ -1704,12 +1772,19 @@ bool CHashFile::Insert(HP_HEAPLENGTH nRecord, UINT32 nHash, void *pRecord)
         m_Cache[iCache].m_hp.GetRange(m_nDirDepth, nStart, nEnd);
         if (iFileDir < nStart || nEnd < iFileDir)
         {
-            Log.WriteString("CHashFile::Insert - Directory points to the wrong page." ENDLINE);
+            Log.tinyprintf("CHashFile::Insert - Directory entry (0x%08X) points to the wrong page (0x%08X-0x%08X)." ENDLINE,
+                iFileDir, nStart, nEnd);
             return false;
         }
         int errInserted = m_Cache[iCache].m_hp.Insert(nRecord, nHash, pRecord);
-        if (IS_HP_SUCCESS(errInserted)) break;
-        if (errInserted == HP_INSERT_ERROR_ILLEGAL)
+        if (IS_HP_SUCCESS(errInserted))
+        {
+            // The record was inserted successfully, so the page is dirty.
+            //
+            m_Cache[iCache].m_iState = HF_CACHE_UNPROTECTED;
+            break;
+        }
+        else if (HP_INSERT_ERROR_ILLEGAL == errInserted)
         {
             return false;
         }
@@ -1730,7 +1805,7 @@ bool CHashFile::Insert(HP_HEAPLENGTH nRecord, UINT32 nHash, void *pRecord)
                 // We have a forked dump in progress, so we will wait until the
                 // child exits.
                 //
-                sleep(1);
+                MuxAlarm.Sleep(time_1s);
             } while (mudstate.dumping);
         }
 #endif // !WIN32
@@ -1797,26 +1872,36 @@ bool CHashFile::Insert(HP_HEAPLENGTH nRecord, UINT32 nHash, void *pRecord)
         m_Cache[iEmpty0].m_iState = HF_CACHE_UNPROTECTED;
         m_Cache[iEmpty1].m_iState = HF_CACHE_UNPROTECTED;
 
-        // Now Flush these pages out.
-        //
-        FlushCache(iEmpty1);
-        FlushCache(iEmpty0);
-#ifdef DO_COMMIT
-#ifdef WIN32
-        FlushFileBuffers(m_hPageFile);
-#else // WIN32
-        fsync(m_hPageFile);
-#endif // WIN32
-#endif // DO_COMMIT
-
-        // Now, update the directory.
+        // Update the directory.
         //
+        m_Cache[iEmpty0].m_hp.GetRange(m_nDirDepth, nStart, nEnd);
+        for ( ; nStart <= nEnd; nStart++)
+        {
+            m_hpCacheLookup[nStart] = iEmpty0;
+        }
         m_Cache[iEmpty1].m_hp.GetRange(m_nDirDepth, nStart, nEnd);
         for ( ; nStart <= nEnd; nStart++)
         {
             m_pDir[nStart] = oNew;
+            m_hpCacheLookup[nStart] = iEmpty1;
         }
 
+        // Flush the pages out.
+        //
+        FlushCache(iEmpty1);
+        FlushCache(iEmpty0);
+
+#ifdef DO_COMMIT
+        if (!mudstate.bStandAlone)
+        {
+#ifdef WIN32
+            FlushFileBuffers(m_hPageFile);
+#else // WIN32
+            fsync(m_hPageFile);
+#endif // WIN32
+        }
+#endif // DO_COMMIT
+
         // Write Directory
         //
 #ifdef WIN32
@@ -1833,14 +1918,16 @@ bool CHashFile::Insert(HP_HEAPLENGTH nRecord, UINT32 nHash, void *pRecord)
 #endif // WIN32
 
 #ifdef DO_COMMIT
+        if (!mudstate.bStandAlone)
+        {
 #ifdef WIN32
-        FlushFileBuffers(m_hDirFile);
+            FlushFileBuffers(m_hDirFile);
 #else // WIN32
-        fsync(m_hDirFile);
+            fsync(m_hDirFile);
 #endif // WIN32
+        }
 #endif // DO_COMMIT
     }
-    m_Cache[iCache].m_iState = HF_CACHE_UNPROTECTED;
     return true;
 }
 
@@ -1850,16 +1937,19 @@ bool CHashFile::DoubleDirectory(void)
     HP_DIRINDEX nNewDirDepth = m_nDirDepth + 1;
 
     HF_PFILEOFFSET pNewDir = (HF_PFILEOFFSET)MEMALLOC(sizeof(HF_FILEOFFSET)*nNewDir);
-    if (ISOUTOFMEMORY(pNewDir))
-    {
-        return false;
-    }
+    ISOUTOFMEMORY(pNewDir);
+
+    int *pNewCacheLookup = new int[nNewDir];
+    ISOUTOFMEMORY(pNewCacheLookup);
 
     unsigned int iNewDir = 0;
     for (unsigned int iDir = 0; iDir < m_nDir; iDir++)
     {
-        pNewDir[iNewDir++] = m_pDir[iDir];
-        pNewDir[iNewDir++] = m_pDir[iDir];
+        pNewDir[iNewDir]   = m_pDir[iDir];
+        pNewDir[iNewDir+1] = m_pDir[iDir];
+        pNewCacheLookup[iNewDir]   = m_hpCacheLookup[iDir];
+        pNewCacheLookup[iNewDir+1] = m_hpCacheLookup[iDir];
+        iNewDir += 2;
     }
 
     // Write out the new directory. It's always larger than
@@ -1869,6 +1959,10 @@ bool CHashFile::DoubleDirectory(void)
 
     MEMFREE(m_pDir);
     m_pDir = pNewDir;
+
+    MEMFREE(m_hpCacheLookup);
+    m_hpCacheLookup = pNewCacheLookup;
+
     m_nDirDepth = nNewDirDepth;
     m_nDir = nNewDir;
     return true;
@@ -1885,8 +1979,7 @@ HP_DIRINDEX CHashFile::FindFirstKey(UINT32 nHash)
         cs_fails++;
         return HF_FIND_END;
     }
-    HF_FILEOFFSET oPage = m_pDir[iFileDir];
-    iCache = ReadCache(oPage, &cs_rhits);
+    iCache = ReadCache(iFileDir, &cs_rhits);
     if (iCache < 0)
     {
         cs_fails++;
@@ -1896,11 +1989,12 @@ HP_DIRINDEX CHashFile::FindFirstKey(UINT32 nHash)
     m_Cache[iCache].m_hp.GetRange(m_nDirDepth, nStart, nEnd);
     if (iFileDir < nStart || nEnd < iFileDir)
     {
-        Log.WriteString("CHashFile::Find - Directory points to the wrong page." ENDLINE);
+        Log.tinyprintf("CHashFile::Find - Directory entry (0x%08X) points to the wrong page (0x%08X-0x%08X)." ENDLINE,
+            iFileDir, nStart, nEnd);
         return HF_FIND_END;
     }
-    unsigned int numchecks;
 
+    unsigned int numchecks;
     HP_DIRINDEX iDir = m_Cache[iCache].m_hp.FindFirstKey(nHash, &numchecks);
 
     if (iDir == HP_DIR_EMPTY)
@@ -1963,18 +2057,10 @@ bool CHashFile::FlushCache(int iCache)
 
 int CHashFile::AllocateEmptyPage(int nSafe, int Safe[])
 {
-again:
-    bool bFoundOldest = false;
-    int iOldest = 0;
-    int iOldestAge = 0;
-    for (int cnt = m_nCache, i = m_iLastEmpty+1; cnt--; i++)
+    int cnt = m_nCache;
+    while (cnt--)
     {
-        // Modulus
-        //
-        if (i >= m_nCache)
-        {
-            i -= m_nCache;
-        }
+        int i = m_iOldest;
 
         bool bExclude = false;
         for (int j = 0; j < nSafe; j++)
@@ -1985,48 +2071,62 @@ again:
                 break;
             }
         }
-        if (!bExclude)
-        {
-            if (m_Cache[i].m_iState == HF_CACHE_EMPTY)
-            {
-                m_iLastEmpty = i;
-                return i;
-            }
 
-            if (bFoundOldest)
+        ResetAge(i);
+
+        if (  !bExclude
+           && FlushCache(i))
+        {
+            if (HF_CACHE_EMPTY != m_Cache[i].m_iState)
             {
-                int iDiff = iOldestAge - m_Cache[i].m_Age;
-                if (iDiff > 0)
+                UINT32 nStart, nEnd;
+                m_Cache[i].m_hp.GetRange(m_nDirDepth, nStart, nEnd);
+                for ( ; nStart <= nEnd; nStart++)
                 {
-                    iOldestAge = m_Cache[i].m_Age;
-                    iOldest = i;
+                    m_hpCacheLookup[nStart] = -1;
                 }
+                m_Cache[i].m_iState = HF_CACHE_EMPTY;
             }
-            else
-            {
-                iOldestAge = m_Cache[i].m_Age;
-                iOldest = i;
-                bFoundOldest = true;
-            }
-        }
-    }
-
-    if (bFoundOldest)
-    {
-        if (FlushCache(iOldest))
-        {
-            m_Cache[iOldest].m_iState = HF_CACHE_EMPTY;
-            return iOldest;
-        }
-        else
-        {
-            m_Cache[iOldest].m_Age = m_iAgeNext++;
-            goto again;
+            return i;
         }
     }
     return -1;
 }
 
+void CHashFile::ResetAge(int iEntry)
+{
+    if (iEntry == m_iOldest)
+    {
+        // Rotate the doubly-linked list to make the oldest entry the
+        // youngest.
+        //
+        m_iOldest = m_Cache[m_iOldest].m_iYounger;
+    }
+    else if (iEntry == m_Cache[m_iOldest].m_iOlder)
+    {
+        // This is already the youngest entry.
+        //
+    }
+    else
+    {
+        // Unlink this entry.
+        //
+        int iYounger = m_Cache[iEntry].m_iYounger;
+        int iOlder   = m_Cache[iEntry].m_iOlder;
+        m_Cache[iYounger].m_iOlder = iOlder;
+        m_Cache[iOlder].m_iYounger = iYounger;
+
+        // Re-link at the young end of the queue.
+        //
+        iYounger = m_iOldest;
+        iOlder   = m_Cache[iYounger].m_iOlder;
+        m_Cache[iEntry].m_iOlder   = iOlder;
+        m_Cache[iEntry].m_iYounger = iYounger;
+        m_Cache[iYounger].m_iOlder = iEntry;
+        m_Cache[iOlder].m_iYounger = iEntry;
+    }
+}
+
 void CHashFile::Tick(void)
 {
     int nCycle = mudconf.check_interval;
@@ -2058,49 +2158,38 @@ void CHashFile::Tick(void)
     }
 }
 
-typedef struct tagCacheLookup
+int CHashFile::ReadCache(UINT32 iFileDir, int *phits)
 {
-    INT32  oPage;
-    UINT32 iCache;
-} CACHELOOKUP, *PCACHELOOKUP;
+    int iCache = m_hpCacheLookup[iFileDir];
+    HF_FILEOFFSET oPage = m_pDir[iFileDir];
 
-int CHashFile::ReadCache(HF_FILEOFFSET oPage, int *phits)
-{
-    UINT32  nHash = CRC32_ProcessInteger(oPage);
-    nHash = nHash % m_nhpCacheLookup;
-
-    int i = m_hpCacheLookup[nHash];
-
-    if (m_Cache[i].m_iState != HF_CACHE_EMPTY && m_Cache[i].m_o == oPage)
+    if (  iCache != -1
+       && m_Cache[iCache].m_iState != HF_CACHE_EMPTY
+       && m_Cache[iCache].m_o == oPage)
     {
-        m_Cache[i].m_Age = m_iAgeNext++;
+        ResetAge(iCache);
         (*phits)++;
-        return i;
+        return iCache;
     }
 
-    for (i = 0; i < m_nCache; i++)
+    if ((iCache = AllocateEmptyPage(0, NULL)) >= 0)
     {
-        if (m_Cache[i].m_iState != HF_CACHE_EMPTY && m_Cache[i].m_o == oPage)
-        {
-            m_hpCacheLookup[nHash] = i;
-            m_Cache[i].m_Age = m_iAgeNext++;
-            (*phits)++;
-            return i;
-        }
-    }
-
-    if ((i = AllocateEmptyPage(0, 0)) >= 0)
-    {
-        m_hpCacheLookup[nHash] = i;
-
-        if (m_Cache[i].m_hp.ReadPage(m_hPageFile, oPage))
+        if (m_Cache[iCache].m_hp.ReadPage(m_hPageFile, oPage))
         {
             //if (m_Cache[i].m_hp.Validate())
             //{
-                m_Cache[i].m_o = oPage;
-                m_Cache[i].m_iState = HF_CACHE_CLEAN;
-                m_Cache[i].m_Age = m_iAgeNext++;
-                return i;
+                m_Cache[iCache].m_o = oPage;
+                m_Cache[iCache].m_iState = HF_CACHE_CLEAN;
+                ResetAge(iCache);
+
+                UINT32 nStart, nEnd;
+                m_Cache[iCache].m_hp.GetRange(m_nDirDepth, nStart, nEnd);
+                for ( ; nStart <= nEnd; nStart++)
+                {
+                    m_hpCacheLookup[nStart] = iCache;
+                }
+
+                return iCache;
             //}
         }
         else
@@ -2275,7 +2364,7 @@ bool CHashTable::DoubleDirectory(void)
             pNewDir[iNewDir++] = m_pDir[iDir];
         }
 
-        delete m_pDir;
+        delete [] m_pDir;
 
         m_pDir = pNewDir;
         m_nDirDepth = nNewDirDepth;
@@ -2385,7 +2474,7 @@ void CHashTable::Final(void)
                 m_hpLast = hp;
             }
         }
-        delete m_pDir;
+        delete [] m_pDir;
         m_pDir = NULL;
     }
 }
@@ -2446,6 +2535,11 @@ HP_DIRINDEX CHashTable::FindNext(HP_PHEAPLENGTH pnRecord, void *pRecord)
     return HF_FIND_END;
 }
 
+INT64 CHashTable::GetEntryCount()
+{
+    return m_nEntries;
+}
+
 void CHashTable::GetStats
 (
     unsigned int *hashsize,
@@ -2470,11 +2564,11 @@ CLogFile Log;
 void CLogFile::WriteInteger(int iNumber)
 {
     char aTempBuffer[20];
-    int nTempBuffer = mux_ltoa(iNumber, aTempBuffer);
+    size_t nTempBuffer = mux_ltoa(iNumber, aTempBuffer);
     WriteBuffer(nTempBuffer, aTempBuffer);
 }
 
-void CLogFile::WriteBuffer(int nString, const char *pString)
+void CLogFile::WriteBuffer(size_t nString, const char *pString)
 {
     if (!bEnabled)
     {
@@ -2487,14 +2581,14 @@ void CLogFile::WriteBuffer(int nString, const char *pString)
 
     while (nString > 0)
     {
-        int nAvailable = SIZEOF_LOG_BUFFER - m_nBuffer;
+        size_t nAvailable = SIZEOF_LOG_BUFFER - m_nBuffer;
         if (nAvailable == 0)
         {
             Flush();
             continue;
         }
 
-        int nToMove = nAvailable;
+        size_t nToMove = nAvailable;
         if (nString < nToMove)
         {
             nToMove = nString;
@@ -2516,7 +2610,7 @@ void CLogFile::WriteBuffer(int nString, const char *pString)
 
 void CLogFile::WriteString(const char *pString)
 {
-    int nString = strlen(pString);
+    size_t nString = strlen(pString);
     WriteBuffer(nString, pString);
 }
 
@@ -2530,7 +2624,7 @@ void DCL_CDECL CLogFile::tinyprintf(char *fmt, ...)
     WriteBuffer(nString, aTempBuffer);
 }
 
-void MakeLogName
+static void MakeLogName
 (
     const char *pBasename,
     const char *szPrefix,
@@ -2623,7 +2717,7 @@ void CLogFile::Flush(void)
         m_nSize += m_nBuffer;
 #ifdef WIN32
         unsigned long nWritten;
-        WriteFile(m_hFile, m_aBuffer, m_nBuffer, &nWritten, NULL);
+        WriteFile(m_hFile, m_aBuffer, (DWORD)m_nBuffer, &nWritten, NULL);
 #else // WIN32
         write(m_hFile, m_aBuffer, m_nBuffer);
 #endif // WIN32
@@ -2933,7 +3027,7 @@ again:
     {
         // Problems.
         //
-    	fprintf(stderr, "We are freeing unallocated pointers (0x%08X) on %s, %d\n", pointer, file, line);
+        fprintf(stderr, "We are freeing unallocated pointers (0x%08X) on %s, %d\n", pointer, file, line);
     }
 }
 
