@@ -1,10 +1,10 @@
-// timeutil.cpp -- CLinearTimeAbsolute, and CLinearTimeDelta modules.
+// timeutil.h -- CLinearTimeAbsolute, and CLinearTimeDelta modules.
 //
-// $Id: timeutil.h,v 1.6 2005/09/27 12:14:39 sdennis Exp $
+// $Id: timeutil.h,v 1.17 2006/01/07 05:31:52 sdennis Exp $
 //
-// MUX 2.3
-// Copyright (C) 1998 through 2003 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.  
+// MUX 2.4
+// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
+// rights not explicitly given are reserved.
 //
 // Date/Time code based on algorithms presented in "Calendrical Calculations",
 // Cambridge Press, 1998.
@@ -98,10 +98,15 @@ private:
 
 public:
     CLinearTimeDelta(void);
+    CLinearTimeDelta(INT64 arg_t100ns);
     CLinearTimeDelta(CLinearTimeAbsolute, CLinearTimeAbsolute);
 
     void ReturnTimeValueStruct(struct timeval *tv);
+#ifdef HAVE_NANOSLEEP
+    void ReturnTimeSpecStruct(struct timespec *ts);
+#endif
     long ReturnMilliseconds(void);
+    INT64 ReturnMicroseconds(void);
     long ReturnDays(void);
     long ReturnSeconds(void);
     char *ReturnSecondsString(int nFracDigits = 0);
@@ -116,6 +121,37 @@ public:
     void operator+=(const CLinearTimeDelta& ltd);
 };
 
+class CMuxAlarm
+{
+private:
+    bool bAlarmSet;
+#ifdef WIN32
+    HANDLE hThread;
+#endif
+
+public:
+    bool bAlarmed;
+
+#ifdef WIN32
+    volatile HANDLE hSemAlarm;
+    volatile DWORD  dwWait;
+    ~CMuxAlarm();
+#endif
+
+    CMuxAlarm(void);
+    void Sleep(CLinearTimeDelta ltd);
+    void SurrenderSlice(void);
+
+    void Set(CLinearTimeDelta ltdDeadline);
+    void Clear(void);
+
+#ifndef WIN32
+    void Signal(void);
+#endif
+};
+
+extern CMuxAlarm MuxAlarm;
+
 #define FACTOR_NANOSECONDS_PER_100NS 100
 #define FACTOR_100NS_PER_MICROSECOND 10
 #define FACTOR_100NS_PER_MILLISECOND 10000
@@ -130,7 +166,20 @@ extern const INT64 FACTOR_100NS_PER_HOUR;
 extern const INT64 FACTOR_100NS_PER_DAY;
 extern const INT64 FACTOR_100NS_PER_WEEK;
 
-void TIME_Initialize(void);
+const CLinearTimeDelta time_250ms = 250*FACTOR_100NS_PER_MILLISECOND;
+const CLinearTimeDelta time_1s    = FACTOR_100NS_PER_SECOND;
+const CLinearTimeDelta time_15s   = 15*FACTOR_100NS_PER_SECOND;
+const CLinearTimeDelta time_30s   = 30*FACTOR_100NS_PER_SECOND;
+const CLinearTimeDelta time_45s   = 45*FACTOR_100NS_PER_SECOND;
+const CLinearTimeDelta time_15m   = 15*FACTOR_100NS_PER_MINUTE;
+const CLinearTimeDelta time_30m   = 30*FACTOR_100NS_PER_MINUTE;
+const CLinearTimeDelta time_1w    = FACTOR_100NS_PER_WEEK;
+
+extern void TIME_Initialize(void);
+#ifdef WIN32
+extern BOOL CalibrateQueryPerformance(void);
+#endif
+
 #ifdef SMALLEST_INT_GTE_NEG_QUOTIENT
 INT64 i64Mod(INT64 x, INT64 y);
 INT64 i64FloorDivision(INT64 x, INT64 y);
@@ -152,5 +201,8 @@ int DCL_INLINE iFloorDivisionMod(int x, int y, int *piMod) \
 
 extern bool ParseDate(CLinearTimeAbsolute &lta, char *pDateString, bool *pbZoneSpecified);
 extern bool isLeapYear(long iYear);
+extern const char *monthtab[];
+extern char *DayOfWeekString[];
+extern int iMod(int x, int y);
 
 #endif // TIMEUTIL_H
